local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")
local L = LibStub("AceLocale-3.0"):GetLocale("ag_Options", true)

aUF.Options.ModOnRegisterOptions = {}
do
	local function setUnitAuraPosition(info, value)
		local class = info[(#info + ((type(info.arg) == "number" and info.arg) or -2))]
		aUF.db.profile.units[class].AuraPos = value
		aUF:CallUnitMethods("UpdateAuras",true,class,"type")
		aUF:CallUnitMethods("AuraPosition",true,class,"type")
	end

	local function setUnitAuraOption(info, value)
		local offset = ((type(info.arg) == "number" and info.arg) or -2)
		local class = info[(#info + offset)]
		aUF.db.profile.units[info[#info + offset]][info[#info]] = value
		aUF:CallUnitMethods("SetAuraCount",nil,class,"type")
		aUF:CallUnitMethods("AuraPosition",nil,class,"type")
		aUF:CallUnitMethods("UpdateAuras",nil,class,"type")
	end

	local function setUnitAuraDebuffColor(info, value)
		local class = info[(#info + ((type(info.arg) == "number" and info.arg) or -2))]
		aUF.db.profile.units[class].AuraDebuffC = value
		aUF:CallUnitMethods("UpdateAuras",true,class,"type")
	end

	local function getUnitAuraFilter(info)
		local class = info[(#info + ((type(info.arg) == "number" and info.arg) or -2))]
		return aUF.db.profile.units[class].AuraFilter == 1
	end

	local function setUnitAuraFilter(info, value)
		local class = info[(#info + ((type(info.arg) == "number" and info.arg) or -2))]
		aUF.db.profile.units[class].AuraFilter = value and 1 or 0
		aUF:CallUnitMethods("UpdateAuras",true,class,"type")
	end

	aUF.Options.ModOnRegisterOptions.Auras = function()
		local config = {
			hidden = function() return not (aUF.modules.Auras:IsEnabled()) end,
			name = L["Auras"],
			type = "group",
			desc = L["Aura options."],
			args = {
				AuraPos = {
					name = L["Position"],
					type = "select",
					desc = L["Sets the position in which auras should be displayed in releation to the unit frame."],
					set = setUnitAuraPosition,
					values = {Right = "Right", Left = "Left", Above = "Above", Below = "Below",Inside = "Inside",Hidden = "Hidden"},
					order = 1,
				},
				Group1 = {
					name = "",
					type = "group",
					inline = true,
					order = 2,
					args = {
						AuraRows = {
							name = L["Rows"],
							type = "range",
							desc = L["Rows of auras to display."],
							min = 1,
							max = 4,
							step = 1,
							set = setUnitAuraOption,
							order = 1,
							arg = -3,
						},
						AuraColumns = {
							name = L["Columns"],
							type = "range",
							desc = L["Columns of auras to display."],
							min = 8,
							max = 24,
							step = 1,
							set = setUnitAuraOption,
							arg = -3,
						},
					},
				},
				Group2 = {
					name = "",
					type = "group",
					inline = true,
					args = {
						AuraDebuffC = {
							name = L["Enable Highlights"],
							type = "toggle",
							desc = L["Highlights the frame in color if debuffed."],
							set = setUnitAuraDebuffColor,
							order = 1,
							arg = -3,
						},
						AuraFilter = {
							name = L["Filter Auras"],
							type = "toggle",
							desc = L["Filters auras relevant to your class"],
							get = getUnitAuraFilter,
							set = setUnitAuraFilter,
							order = 2,
							arg = -3,
						},
						AuraPreferBuff = {
							name = L["Prioritize Buffs"],
							type = "toggle",
							desc = L["Prioritize buffs above debuffs"],
							set = setUnitOption,
							order = 3,
							arg = -3,
						},
						cooldown = {
							name = L["Enable Cooldown"],
							type = "toggle",
							desc = L["Shows a cooldown dial on auras"],
							set = setUnitAuraOption,
							arg = -3,
						},			
					},
				},
			}
		}
		return config
	end
end

do
	function aUF.Options.ModOnRegisterOptions.Healthbar()
		local config = {
			inline = true,
			name = L["Healthbar"],
			desc = L["Healthbar options."],
			type = 'group',
			args = {
				ClassColorBars = {
					name = L["Color By Class"],
					type = "toggle",
					desc = L["Colors the healthbar by class."],
					set = "LayoutSet",
					arg = -3,
				},
				TargetShowHostile = {
					name = L["Color By Alignment"],
					type = "toggle",
					desc = L["Colors the healthbar by unit alignment towards you."],
					set = "LayoutSet",
	--				hidden = function() local class = class return class ~= "target" end,
					arg = -3,
				},
				FadeHealth = {
					name = L["Fade Healthbar"],
					type = "toggle",
					desc = L["Fades healthbar changes."],
					set = "LayoutSet",
					arg = -3,
				},
				SmoothHealth = {
					name = L["Smooth Healthbar"],
					type = "toggle",
					desc = L["Smooths healthbar changes."],
					set = "LayoutSet",
					arg = -3,
				},
				AggroHealth = {
					name = L["Color Red at Aggro"],
					type = "toggle",
					desc = L["Color the healthbar Red when the unit has aggro."],
					set = "LayoutSet",
					arg = -3,
				},
			}
		}
		return config
	end
end

do
	function aUF.Options.ModOnRegisterOptions.Powerbar()
		local config = {
			inline = true,
			hidden = function() return not (aUF.modules.Powerbar:IsEnabled()) end,
			name = L["Powerbar"],
			desc = L["Powerbar options."],
			type = 'group',
			args = {
				ShowMana = {
					name = L["Enable Powerbar"],
					type = "toggle",
					desc = L["Enables the powerbar."],
					set = "LayoutSet",
					arg = -3,
				},
				FadeMana = {
					name = L["Fade Powerbar"],
					type = "toggle",
					desc = L["Fades powerbar changes."],
					set = "LayoutSet",
					arg = -3,
				},
				SmoothMana = {
					name = L["Smooth Powerbar"],
					type = "toggle",
					desc = L["Smooth powerbar changes."],
					set = "LayoutSet",
					arg = -3,
				},
			},
		}
		return config
	end
end

do
	function aUF.Options.ModOnRegisterOptions.Highlight()
		local config = {
			hidden = function() return not (aUF.modules.Highlight:IsEnabled()) end,		
			name = L["Highlight"],
			desc = L["Highlight options."],
			type = 'group',
			args = {
				HighlightSelected = {
					name = L["Enable Highlights"],
					type = "toggle",
					desc = L["Highlights the frame when selected."],
					set = "UpdateSet",
				}
			}
		}
		return config
	end
end

do
	function aUF.Options.ModOnRegisterOptions.Combattext()
		local config = {
			hidden = function() return not (aUF.modules.Combattext:IsEnabled()) end,		
			name = L["Combat Text"],
			desc = L["Combat text options."],
			type = 'group',
			args = {
				ShowCombat = {
					name = L["Enable Combat Text"],
					type = "toggle",
					desc = L["Displays a text overlay with combat information."],
					set = "LayoutSet",
				}	
			}
		}
		return config
	end
end

do
	function aUF.Options.ModOnRegisterOptions.Castbar()
		local config = {
			inline = true,
			hidden = function() return not (aUF.modules.Castbar:IsEnabled()) end,		
			name = L["Castbar"],
			desc = L["Castbar options."],
			type = 'group',
			args = {
				CastBar = {
					name = L["Enable Castbar"],
					type = "toggle",
					desc = L["Displays a casting bar for the unit."],
					set = "LayoutSet",
					arg = -3,
				},	
			},
		}
		return config
	end
end

do
	function aUF.Options.ModOnRegisterOptions.Portrait()
		config = {
			hidden = function() return not (aUF.modules.Portrait:IsEnabled()) end,		
			name = L["Portrait"],
			desc = L["Portrait options."],
			type = 'group',
			args = {
				Portrait = {
					name = L["Enable Portrait"],
					type = "toggle",
					desc = L["Draws a portrait of the character."],
					set = "LayoutSet",
				},
				PortraitStyle = {
					name = L["Portrait Style"],
					type = "select",
					desc = L["How to display the portrait."],
					values = {
						["3d"] = L["3d portrait"],
						["2d"] = L["2d portrait"],
						["icon"] = L["Class icon"],
					},
					set = "LayoutSet",
				},
			},
		}
		return config
	end
end

do
	function aUF.Options.ModOnRegisterOptions.Rangecheck()
		local config = {
			hidden = function() return not (aUF.modules.Rangecheck:IsEnabled()) end,		
			name = L["Range Check"],
			desc = L["Range check options."],
			type = 'group',
			args = {
				rangeCheck = {
					name = L["Enable Range Check"],
					type = "toggle",
					desc = L["Fades out of range units."],
					set = "UpdateSet",
				},	
			},
		}
		return config
	end
end

do
	function aUF.Options.ModOnRegisterOptions.Xprepbar()
		local config = {
			inline = true,
			hidden = function(info) return (not (aUF.modules.Xprepbar:IsEnabled() and aUF.modules.Xprepbar.inherit[info[#info-2]])) end,
			name = L["Experience Bar"],
			desc = L["Experience options."],
			type = 'group',
			args = {
				ShowXP = {
					name = L["Enable Experiencebar"],
					type = "toggle",
					desc = L["Enables the Experiencebar."],
					set = "LayoutSet",
					arg = -3,
					hidden = false,
				},
				ShowXPTooltip = {
					name = L["Show Tooltip"],
					type = "toggle",
					desc = L["Show tooltip when mousing over the bar"],
					set = "LayoutSet",
					arg = -3,
					hidden = false,
				},
			},
		}
		return config
	end
end
