local L = LibStub("AceLocale-3.0"):GetLocale("ag_Options", true)
local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")

local SM = LibStub("LibSharedMedia-3.0")

local options = {}
local moduleOptions = {}
-----------------------------------------------------------------------------------------------------------------------------------------------------
---- getter / setter functions ----------------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------------------------------------

local function getClass(info,offset)
    return info[(#info + ((type(offset) == "number" and offset) or -2))]
end

local function getOption(info)
	return aUF.db.profile[info[#info]]
end

local function setOption(info, value)
	aUF.db.profile[info[#info]] = value
end

local function getColor(info)
	local key = info.arg
	if type(key) == "string" then
		return aUF.db.profile[key].r, aUF.db.profile[key].g, aUF.db.profile[key].b, aUF.db.profile[key].a
	else
		return aUF.db.profile.ManaColor[key].r, aUF.db.profile.ManaColor[key].g, aUF.db.profile.ManaColor[key].b, aUF.db.profile.ManaColor[key].a
	end
end

local function setColor(info, r, g, b, a)
	local key = info.arg
	if type(key) == "string" then
		aUF.db.profile[key] = { r = r, g = g, b = b, a = a }
	else
		aUF.db.profile.ManaColor[key] = { r = r, g = g, b = b, a = a }
	end
	if key == "PartyFrameColors" or key == "TargetFrameColors" or key == "FrameBorderColors" then
		aUF:CallUnitMethods("BorderBackground")
		aUF:CallUnitMethods("BorderBackground",nil,nil,nil,"subgroups")
	else
		aUF:CallUnitMethods("StatusBarsColor")
	end
end

local function setUnitFrameStyle(info, value)
    local class = getClass(info,info.arg)
	
	aUF:CallUnitMethods("UnapplyLayout",true,class,"type")
	
	aUF.db.profile.units[class].FrameStyle = value
		
	aUF:CallUnitMethods("ApplyLayout",true,class,"type")
	aUF:CallUnitMethods("BorderBackground",nil,class,"type")
	aUF:CallUnitMethods("UpdateAll",nil,class,"type")
	for k,v in pairs(aUF.subgroups) do
		v:UpdateWidth()
	end
    options:UpdateStringOptions(class)
end

local function setUnitBorderStyle(info, value)
    local class = getClass(info,info.arg)
	aUF.db.profile.units[class].BorderStyle = value
	aUF:CallUnitMethods("BorderBackground",nil,class,"type")
end

local function setUnitWidth(info, value)
    local class = getClass(info,info.arg)
	aUF.db.profile.units[class].Width = value
	aUF:CallUnitMethods("ApplyLayout",nil,class,"type")
	for k,v in pairs(aUF.subgroups) do
		v:UpdateWidth()
	end
end

local function setUnitHeight(info, value)
    local class = getClass(info,info.arg)
	aUF.db.profile.units[class].Height = value
	aUF:CallUnitMethods("ApplyLayout",nil,class,"type")
end

local function setUnitScale(info, value)
    local class = getClass(info,info.arg)
	aUF.db.profile.units[class].Scale = value
	aUF:CallUnitMethods("LoadScale",nil,class,"type")
	aUF:CallUnitMethods("LoadPosition",nil,class,"type")
	for k,v in pairs(aUF.subgroups) do
		v:UpdateScale()
		v:LoadPosition()
	end
end

local function setFontOption(info, value)
    local class = getClass(info,info.arg)
    local tbl = SM:List("font")
	aUF.db.profile.units[class][info[#info]] = tbl[value]
	aUF:CallUnitMethods("ApplyLayout",nil,class,"type")
	aUF:CallUnitMethods("UpdateAll",nil,class,"type")
end

local function setGroupOption(info, value)
	local num = -2
    if info.arg and type(info.arg) == "number" then
		num = info.arg
	end
    aUF.db.profile.units[info[#info+num]][info[#info]] = value
    aUF:UpdatePartyGrouping(info[#info+num])
end

local function getFrameOptions(info)
	local class = info[#info-3]
	local frame = info[#info-1]
	local dbPath = aUF.db.profile.units[class][info[#info]]
	return (dbPath and dbPath[frame]) or 1
end

local function setFrameOptions(info, value)
	local class = info[#info-3]
	local frame = info[#info-1]
	local dbPath = aUF.db.profile.units[class][info[#info]]
	if not dbPath then
		aUF.db.profile.units[class][info[#info]] = {}
		dbPath = aUF.db.profile.units[class][info[#info]]
	end
	dbPath[frame] = value
	aUF:CallUnitMethods("SetHeight",nil,class,"type")
end

local function getStringTags(info, tag)
	local class = info[#info-3]
	local string = info[#info-1]
	
	local dbPath = aUF.layouts[aUF.db.profile.units[class].FrameStyle].db.profile[class][string]
	
	if dbPath and string.find(dbPath, tag) then
		return true
	end
end

local function textSort(alpha, bravo)
	local tag1 = aUF.textFunctions[alpha] 
	local tag2 = aUF.textFunctions[bravo]
	return (tag1.order or 0) < (tag2.order or 0)
end
							
local function setStringTags(info, value, state)
	local class = info[#info-3]
	local key = info[#info-1]

	local strings = aUF.layouts[aUF.db.profile.units[class].FrameStyle].db.profile[class]
	
	if not (type(strings[key]) == "string") then
		strings[key] = ""
	end
	
	local sortTbl = {}
	for word in string.gmatch(strings[key], "%a+") do
		if not (word == value) then
			table.insert(sortTbl, word)
		end
	end
	if state then
		table.insert(sortTbl, value)
	end
	
	table.sort(sortTbl, textSort)
	strings[key] = table.concat(sortTbl, " ")
	
	local frame
	for k,v in pairs(aUF.units) do
		if v.type == class then
			local frame = v.strings[key]	
			v:UnregisterString(frame,key)
			v:RegisterString(frame,strings[key],key)			
			v:UpdateTextString(frame)
		end
	end	
end

local function setHideInRaid(info, value)
    if info.arg and type(info.arg) == "number" then
		aUF.db.profile[info[#info]] = value
	else
		aUF.db.profile[info[#info]] = value
    end
	aUF:RAID_ROSTER_UPDATE(true)
end

local function classOrder(class)
	local order
	if class:find("^player") then
		order = 50 + class:len()/1000
	elseif class:find("^pet") then
		order = 51 + class:len()/1000
	elseif class:find("^target") then
		order = 52 + class:len()/1000
	elseif class:find("^focus") then
		order = 53 + class:len()/1000		
	elseif class:find("^party") then
		order = 54 + class:len()/1000
	elseif class:find("^partypet") then
		order = 55 + class:len()/1000		
	elseif class:find("^raid") then
		order = 56 + class:len()/1000
	elseif class:find("^raidpet") then
		order = 55 + class:len()/1000
	else
		class = 58 + class:len()/1000
	end
	return order
end

options.table = {
	type = "group",
	icon = "",
	name = "ag_UnitFrames",
	childGroups = "tree",
	args = {
	}
}

-----------------------------------------------------------------------------------------------------------------------------------------------------
---- on the run updates -----------------------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------------------------------------

local unitOptionsTable	
local raidGroupOptions	
local groupOptions	
function options:AddUnitOptions(class)
	local path = options.table.args.Units
	path.args[class] = unitOptionsTable

	if class == "raid" then
		options:UpdateSubgroups()
	end
end

local stringValues
local tagOptions

function options:UpdateStringOptions(class)
	local path = options.table.args.Units
	local bars = aUF.SetupStrings[class]
	if bars then
		for k,v in pairs(bars) do
			if not path.args[class].args.Text.args[k] then
				path.args[class].args.Text.args[k] = tagOptions
			end
		end
	end
end

-----------------------------------------------------------------------------------------------------------------------------------------------------
---- populate enabled units/modules -----------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------------------------------------

local xpetGroupTable = {parent = L["Parent"], free = L["Free"], self = L["Self"]}
local xtargetGroupTable = {parent = L["Parent"], free = L["Free"]}
local partyGroupTable = {free = L["Free"], self = L["Self"]}

local function getGroupOtion(info)
	local raidClass = info[#info+((info.arg or 0) - 1)]
	local class = info[#info+(info.arg or 0)]
	if raidClass == "raid" then
		local group = info[#info+((info.arg or 0) + 1)]
		return (aUF.db.profile.subgroups[group] and aUF.db.profile.subgroups[group][info[#info]]) or nil
	elseif aUF.partyUnits[class] then
		return (aUF.db.profile[class.."group"] and aUF.db.profile[class.."group"][info[#info]]) or nil
	end
end

local function setGroupOtion(info, value)
	local raidClass = info[#info+((info.arg or 0) - 1)]
	local class = info[#info+(info.arg or 0)]
	if raidClass == "raid" then
		local group = info[#info+((info.arg or 0) + 1)]
		if not (aUF.db.profile.subgroups[group]) then
			aUF.db.profile.subgroups[group] = {}
		end
		aUF.db.profile.subgroups[group][info[#info]] = value
		aUF.subgroups[group]:UpdateAll()
	elseif aUF.partyUnits[class] and aUF.db.profile[class.."group"] then
		aUF.db.profile[class.."group"][info[#info]] = value
		if aUF.subgroups[class] then
			aUF.subgroups[class]:UpdateAll()
		end
	end
end

local groupHidden = function(info) 
	local class = info[#info+(info.arg or -3)]
	if not (class == "raid") then
		return true
	end
end

-- grouping
raidGroupOptions = {
	name = L["Groups"],
	type = "group",
	desc = L["Configuration of raid subgroups."],
	order = 3,
	childGroups = "tree",
	args = {
		New = {
			name = L["New Group"],
			type = "execute",
			desc = L["Creates a new raid subgroup."],
			func = function(info) 
				aUF:CreateNewGroup()
				aUF.Options:UpdateSubgroups()
			end,
			order = 1,
			hidden = false,
		}
	},
	plugins = {},
	hidden = groupHidden,
	arg = -1,
}

groupOptions = {
	name = function(info) if info[#info] == "GroupOptions" then return L["Group"] else return (aUF.subgroups[info[#info]] and aUF.subgroups[info[#info]].database and aUF.subgroups[info[#info]].database.Name) or info[#info] end end,
	type = "group",
	desc = L["Group Options."],
	order = 3,
	args = {
		Name = {
			name = L["Group Name"],
			type = "input",
			desc = L["Sets the name of group, displayed in the anchor."],
			get = getGroupOtion,
			set = setGroupOtion,
			order = 2,
			arg = -2,
			hidden = groupHidden,
		},	
		ShowAnchor = {
			name = L["Show Group Header"],
			type = "toggle",
			desc = L["Should the group header be shown or not."],
			get = getGroupOtion,
			set = setGroupOtion,
			order = 1,
			arg = -2,
			hidden = false,
		},
		Group1 = {
			name = "",
			type = "group",
			inline = true,
			order = 3,
			hidden = false,
			args = {
				AnchorOffset = {
					name = L["Anchor Offset"],
					type = "range",
					desc = L["Sets the group offset from the anchor."],
					get = getGroupOption,
					set = setGroupOtion,
					order = 2,
					arg = -3,
					min = 0,
					max = 25,
					step = 1,
					hidden = false,
				},
				Padding = {
					name = L["Padding"],
					type = "range",
					desc = L["Sets the distance between group members."],
					get = getGroupOtion,
					set = setGroupOtion,
					order = 3,
					arg = -3,
					min = 0,
					max = 50,
					step = 1,
					hidden = false,
				},
			}
		},
		Group2 = {
			name = "",
			type = "group",
			inline = true,
			order = 4,
			hidden = false,		
			args = {		
				Grow = {
					name = L["Grow Direction"],
					type = "select",
					desc = L["Controls which way the group grows."],
					values = {LEFT = L["Left3"], RIGHT = L["Right3"], BOTTOM = L["Up"], TOP = L["Down"]},
					get = getGroupOtion,
					set = setGroupOtion,
					order = 3,
					arg = -3,
					hidden = false,
				}
			}
		},
		Group3 = {
			name = "",
			type = "group",
			inline = true,
			order = 5,
			hidden = groupHidden,
			args = {			
				groupFilter = {
					name = L["Group Filter"],
					type = "input",
					desc = L["Sets which units the group should show. For example '1,2,3' or 'MAGE,WARLOCK'. Seperate with commas, no spaces."],
					get = getGroupOtion,
					set = setGroupOtion,
					order = 4,
					arg = -3,
					hidden = false,
				},
				nameList = {
					name = L["Name List"],
					type = "input",
					desc = L["Sets which players the group should show. For example 'Kristiano,Tritonus,Giustiniano'. Seperate with commas, no spaces."],
					get = getGroupOtion,
					set = setGroupOtion,
					order = 5,
					arg = -3,
					hidden = false,
				},
				groupBy = {
					name = L["Sort By"],
					type = "select",
					desc = L["Sorts the group by name, index, subgroup or class."],
					values = {NAME = L["Name"], INDEX = L["Index"], CLASS = L["Class"], GROUP = L["Subgroup"]},
					get = getGroupOtion,
					set = setGroupOtion,
					order = 6,
					arg = -3,
					hidden = false,
				}				
			}
		},
		Delete = {
			name = L["Delete Group"],
			type = "execute",
			desc = L["Deletes the current raid subgroup."],
			func = function(info) if aUF.subgroups[info[#info-1]] then aUF.subgroups[info[#info-1]]:Delete() end end,
			order = 10,
			hidden = function(info) 
				local class = info[#info-3]
				if not (class == "raid") then
					return true
				end
			end,
		}		
	},
	plugins = {},
	hidden = function(info)
		local raidClass = info[#info-2]
		local class = info[#info+(info.arg or -1)]
		if not (class == "party" or class == "partypet") and not (raidClass == "raid") then
			return true
		end
		if raidClass == "raid" then
			local group = info[#info]
			if not (aUF.db.profile.subgroups[group] and aUF.db.profile.subgroups[group].Exists) then
				return true
			end
		elseif not (aUF.db.profile.units[class].GroupWith == "self") then
			return true
		end
	end,
}
function options:PopulateOptions()
	tagOptions = {
		hidden = function(info)
			local class = info[#info-2]
			local frame = info[#info]
			local bars = aUF.SetupStrings[class]
			if aUF.SetupStrings[class] and not (bars[frame]) then
				return true
			end
		end,
		inline = false,
		type = "group",
		name = function(info) return info[#info] end,
		desc = "desc",
		args = {
			Tags = {
				name = "",
				type = "multiselect",
				desc = L["Configures what text the string should display."],
				get = getStringTags,
				set = setStringTags,
				hidden = false,
				disabled = false,
				values = function()
					if not stringValues then
						stringValues = {}
						for k, v in pairs(aUF.textFunctions) do
							stringValues[k] = v.name
						end
					end
					return stringValues
				end,
			}
		}
	}
	
	unitOptionsTable = {
		name = function(info)
			local class = L[info[#info]]
			return class
		end,
		type = "group",
		desc = "desc",
		order = function(info) return classOrder(info[#info]) end,
		childGroups = "tab",
		hidden = function(info) return not(aUF.db.profile.units[info[#info]].enabled) end,
		get = function(info)
            if info.arg and type(info.arg) == "number" then
                return aUF.db.profile.units[info[#info+info.arg]][info[#info]]
            else
                return aUF.db.profile.units[info[#info-2]][info[#info]]
            end
		end,
		set = function(info, value)
            if info.arg and type(info.arg) == "number" then
                aUF.db.profile.units[info[#info+info.arg]][info[#info]] = value
            else
                aUF.db.profile.units[info[#info-2]][info[#info]] = value
            end
		end,
        handler = {
            UpdateSet = function(self, info, value)
                local class = info[(#info + ((type(info.arg) == "number" and info.arg) or -2))]
                aUF.db.profile.units[class][info[#info]] = value
				aUF:CallUnitMethods("UpdateAll",nil,class,"type")
            end,
            LayoutSet = function(self, info, value)
                local class = info[(#info + ((type(info.arg) == "number" and info.arg) or -2))]
				
                aUF.db.profile.units[class][info[#info]] = value
				aUF:CallUnitMethods("ApplyLayout",nil,class,"type")
				aUF:CallUnitMethods("UpdateAll",nil,class,"type")
            end
        },
		plugins = {},
		args = {
			Header = {
				type = 'header',
				order = 1,
				name = function(info)
					local class = L[info[#info-1]]
					return class
				end,
			},
			FrameStyle = {
				hidden = false,
				name = L["Frame"],
				type = "group",
				order = 1,
				args = {
					Group1 = {
						name = "",
						type = "group",
						inline = true,
						order = 1,
						args = {
							HidePartyInRaid = {
								name = L["Hide in Raid"],
								type = "toggle",
								desc = L["Hides party frames in raid."],
								get = getOption,
								set = setHideInRaid,
								order = 1,
								arg = -3,
								hidden = function(info) return not (info[#info-3] == "party") end,
							},
							FiveMan = {
								name = L["Show 5-man Raid as Party"],
								type = "toggle",
								desc = L["Handles as 5-man raid as if it was a normal party."],
								get = getOption,
								set = setHideInRaid,
								order = 1,
								arg = -3,
								hidden = function(info) return not (info[#info-3] == "party") end,
							},								
						},
					},
					Group2 = {
						name = "",
						type = "group",
						inline = true,
						order = 2,
						args = {
							FrameStyle = {
								name = L["Frame Style"],
								type = "select",
								desc = L["Adjust the style of the frame."],
								values = function() local layouts = {} for k, v in pairs(aUF.layouts) do if v:IsEnabled() then layouts[k] = k end end return layouts end,
								set = setUnitFrameStyle,
								order = 2,
								arg = -3,
							},
							BorderStyle = {
								name = L["Border"],
								type = "select",
								desc = L["Adjust the border type."],
								values = {RoundBlizz = "RoundBlizz", SquareBlizz = "SquareBlizz", SquareClean = "SquareClean", Hidden = "Hidden"},
								set = setUnitBorderStyle,
								order = 3,
								arg = -3,
							},
						},
					},
					Group3 = {
						name = "",
						type = "group",
						inline = true,
						order = 3,
						args = {					
							Width = {
								name = L["Width"],
								type = "range",
								desc = L["Adjust the frame width."],
								min = 50,
								max = 400,
								step = 1,
								set = setUnitWidth,
								order = 5,
								arg = -3,
							},
							Height = {
								name = L["Height"],
								type = "range",
								desc = L["Adjust the frame height."],
								min = 10,
								max = 200,
								step = 1,
								set = setUnitHeight,
								order = 6,
								arg = -3,
							},
							Scale = {
								name = L["Scale"],
								type = "range",
								desc = L["Adjust the scale of the frame."],
								min = 0.5,
								max = 2,
								step = 0.05,
								isPercent = true,
								set = setUnitScale,
								order = 7,
								arg = -3,
							},
						},
					},
					Group4 = {
						name = "",
						type = "group",
						inline = true,
						order = 4,
						args = {	
							GroupWith = {
								name = L["Group With"],
								type = "select",
								desc = L["Configures how these unit frames group."],
								values = function(info) 
										if string.find(info[#info-3], "%a+pet") then
											return xpetGroupTable
										elseif string.find(info[#info-3], "%a+target") then
											return xtargetGroupTable
										elseif info[#info-3] == "party" then
											return partyGroupTable
										end
									end,
								set = setGroupOption,
								order = 1,
								hidden = function(info) return not (aUF.partyUnits[info[#info-3]]) end,
								arg = -3,
							},
							PetPos = {
								name = L["Relative Position"],
								type = "select",
								desc = L["Sets the position of the child unit in relation to the parent frame."],
								values = { L["Left Top"], L["Left2"], L["Left Bottom"], L["Right Top"], L["Right2"], L["Right Bottom"], L["Above Left"], L["Above2"], L["Above Right"], L["Below Left"], L["Below2"], L["Below Right"]},
								set = setGroupOption,
								order = 1,
								hidden = function(info) return (not (aUF.partyUnits[info[#info-3]])) or ((not(string.find(info[#info-3],"pet"))) and (not(string.find(info[#info-3],"target")))) end,
								disabled = function(info) if not (aUF.db.profile.units[info[#info-3]].GroupWith == "parent") then return true end end,
								arg = -3,
							},							
						},
					},
				},
			},
			Text = {
				hidden = false,
				name = L["Text"],
				type = "group",
				desc = "desc",
				order = 2,
				childGroups = "select",
				args = {
					fontOverride = {
						name = L["Font Override"],
						type = "toggle",
						desc = L["Enables overriding of font type/size."],
						set = "LayoutSet",
						order = 1,
					},
					Group1 = {
						name = "",
						type = "group",
						inline = true,
						order = 2,
						args = {
							fontOverrideType = {
								name = L["Font Type"],
								type = "select",
								desc = L["Sets the override font type."],
								values = SM:List("font"),
								get = function(info)
									local tbl = SM:List("font")
									for k, v in pairs(tbl) do
										if aUF.db.profile.units[info[#info-3]][info[#info]] == v then
											return k
										end
									end
								end,
								set = setFontOption,
								order = 1,
								arg = -3,
								disabled = function(info) if not (aUF.db.profile.units[info[#info-3]].fontOverride) then return true end end,
							},
							fontOverrideSize = {
								name = L["Font Size"],
								type = "range",
								desc = L["Sets the override font size."],
								min = -10,
								max = 10,
								step = 1,
								set = "LayoutSet",
								arg = -3,
								disabled = function(info) if not (aUF.db.profile.units[info[#info-3]].fontOverride) then return true end end,
							},
						},
					},
					StringText = {
						order = 3,
						name = L["Text Customization"],
						type = "header",
						desc = L["Customizes the text on the unit frames."],
					},
					RaidColorName = {
						order = 4,
						name = L["Color Name By Class"],
						desc = L["Color the unit's name with its raid class color."],
						set = "LayoutSet",
						type = "toggle",
					},
				},
			},
			Bars = {
				hidden = false,
				name = L["Bars"],
				type = "group",
				desc = "desc",
				order = 3,
				args = {
				},
				plugins = {},
				disabled = function(info)
					local class = info[#info-1]
					
				end,
			},
			GroupOptions = groupOptions,
			RaidGroupOptions = raidGroupOptions
		}
	}
	
	options.table.args = {
		Config = {
			name = "config",
			type = "execute",
			order = 9,
			desc = L["Opens the ag_UnitFrames configuration dialog."],
			func = function() LibStub("AceConfigDialog-3.0"):Open("ag_UnitFrames") end,
			guiHidden = true,
			dialogHidden = true,
			dropdownHidden = true,
		},
		Units = {
			name = L["Unit Frames"],
			type = "group",
			order = 10,
			args = {
				FrameColors = {
					order = 2,
					inline = true,
					type= "group",
					name = L["Frame Colors"],
					get = getColor,
					set = setColor,
					args = {
						partybg = {
							name = L["Frames"],
							type = "color",
							desc = L["Change the unit frame background colour."],
							hasAlpha = true,
							arg = "PartyFrameColors",
							order = 1,
						},
						borderbg = {
							name = L["Borders"],
							type = "color",
							desc = L["Change the frame border color."],
							hasAlpha = true,
							arg = "FrameBorderColors",
							order = 2,
						},
					},
				},
				BarColors = {
					type= "group",
					name = L["Bar Colors"],
					get = getColor,
					set = setColor,
					order = 3,
					inline = true,
					args = {
						Health = {
							name = L["Health"],
							type = "color",
							desc = L["Sets the health color."],
							arg = "HealthColor",
							order = 1,
						},
						Mana = {
							name = L["Mana"],
							type = "color",
							desc = L["Sets the mana color."],
							arg = 0,
							order = 2,
						},
						Rage = {
							name = L["Rage"],
							type = "color",
							desc = L["Sets the rage color."],
							arg = 1,
							order = 3,
						},
						Energy = {
							name = L["Energy"],
							type = "color",
							desc = L["Sets the energy color."],
							arg = 3,
							order = 4,
						},
						PetFocus = {
							name = L["Pet Focus"],
							type = "color",
							desc = L["Sets the pet focus color."],
							arg = 2,
							order = 5,
						},
						Casting = {
							type = "color",
							name = L["Casting Bar"],
							desc = L["Sets the color for the casting bar progress."],
							arg = "CastbarColor",
							order = 6,
						},
						Experience = {
							type = "color",
							name = L["Experience"],
							desc = L["Sets the xp color."],
							arg = "XPColor",
							order = 7,
						},
						Rested = {
							type = "color",
							name = L["Rested Experience"],
							desc = L["Sets the rested xp color."],
							arg = "RestedXPColor",
							order = 8,
						},
					},
				},
				Misc = {
					type= "group",
					name = L["Misc Options"],
					get = getColor,
					set = setColor,
					order = 4,
					inline = true,
					args = {
						Locked = {
							order = 10,
							name = L["Lock Frames"],
							type = "toggle",
							desc = L["Locks the unit frames."],
							get = getOption,
							set = setOption,
						},
						BarStyle = {
							order = 5,
							name = L["Bar Texture"],
							type = "select",
							desc = L["Selects the global status bar texture."],
							get = function(info)
								local tbl = SM:List("statusbar")
								for k, v in pairs(tbl) do
									if aUF.db.profile[info[#info]] == v then
										return k
									end
								end
							end,
							set = function(key, value)
								local tbl = SM:List("statusbar")
								aUF.db.profile.BarStyle = tbl[value]
								aUF:CallUnitMethods("ExecModuleMethods","OnSetBarTexture")
							end,
							values = SM:List("statusbar"),
						},
						BarFadeTime = {
							order = 6,
							name = L["Bar Fade Time"],
							type = "range",
							desc = L["Sets the time it takes for a bar to fade"],
							min = 0.1,
							max = 2,
							step = 0.1,
							get = getOption,
							set = setOption,
						},
					},
				},
			},
		},
		Modules = {
			name = "Modules",
			type = "group",
			order = 20,
			args = {},
		},
		Profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(aUF.db),
	}

	options:UpdateSubgroups()
	
	local path = options.table.args.Units
	local values = {}

	path.args.EnabledUnits = {
		order = 1,
		name = L["Enabled Units"],
		desc = L["Sets which units you want frames created for."],
		type = "multiselect",
		get = function(info, key)
			return aUF.db.profile.units[key].enabled
		end,
		set = function(info, key, value)
			aUF.db.profile.units[key].enabled = value
			aUF:LoadUnit(key)
			if not (value) then
				aUF:DisableAllFrames(key)
			else
				aUF:EnableAllFrames(key)
			end
			aUF:RAID_ROSTER_UPDATE(true)
		end,
	}
	for k in pairs(aUF.soloUnits) do
		values[k] = L[k]
	end
	for k in pairs(aUF.partyUnits) do
		values[k] = L[k]
	end
	for k in pairs(aUF.raidUnits) do
		values[k] = L[k]
	end
	
	path.args.EnabledUnits.values = values

	local path = options.table.args.Modules
	local values = {}

	path.args.EnabledModules = {
		name = L["Enabled Modules"],
		desc = L["Enable or disable this module"],
		type = "multiselect",
		get = function(info, modname)
            local module = aUF.modules[modname]
			return module.enabledState
		end,
		set = function(info, modname)
			local module = aUF.modules[modname]
			if module.enabledState then
                aUF.db.profile.modulesDisabled[module.name] = true
				module:Disable()
			else
                aUF.db.profile.modulesDisabled[module.name] = false
				module:Enable()
			end
		end,
	}
	for k,v in pairs(aUF.modules) do
		values[k] = k
	end
	path.args.EnabledModules.values = values

	for k,v in pairs(aUF.modules) do
		local name = v.name
		local tbl
		if not moduleOptions[name] and v.OnRegisterOptions then
			moduleOptions[name] = v:OnRegisterOptions()
		elseif not moduleOptions[name] and self.ModOnRegisterOptions and self.ModOnRegisterOptions[k] then
			moduleOptions[name] = self.ModOnRegisterOptions[k]()
		end
		tbl = moduleOptions[name]
		local subConfig = v.subConfig
		if subConfig then
			unitOptionsTable.args[subConfig].plugins[name] = {}
			unitOptionsTable.args[subConfig].plugins[name][name] = tbl
		else
			unitOptionsTable.plugins[name] = {}
			unitOptionsTable.plugins[name][name] = tbl
		end
	end

	for k in pairs(aUF.soloUnits) do
		if aUF.db.profile.units[k].enabled then
			aUF.Options:AddUnitOptions(k)
			aUF.Options:UpdateStringOptions(k)
		end
	end
	for k in pairs(aUF.partyUnits) do
		if aUF.db.profile.units[k].enabled then
			aUF.Options:AddUnitOptions(k)
			aUF.Options:UpdateStringOptions(k)
		end
	end
	for k in pairs(aUF.raidUnits) do
		if aUF.db.profile.units[k].enabled then
			aUF.Options:AddUnitOptions(k)
			aUF.Options:UpdateStringOptions(k)
		end
	end
	options.loaded = true
end


--options loading
local opened
function options:OpenOptions()
	if not (opened) then
		LibStub("AceConfig-3.0"):RegisterOptionsTable("ag_UnitFrames", aUF.Options.table)
		LibStub("AceConfigDialog-3.0"):SetDefaultSize("ag_UnitFrames", 810, 550)
		options:PopulateOptions()
		opened = true
	end
	LibStub("AceConfigDialog-3.0"):Open("ag_UnitFrames") 
end

local blizzTbl = {
	name = L["Options Frame"],
	desc = "hat",
	type = "group",
	args = {
		button = {
			name = L["Open"],
			desc = L["Opens the ag_UnitFrames configuration dialog."],
			type = "execute",
			func = function() options:OpenOptions() end,
		},
	},
}

local loadedBlizz
function options:CreateBlizzPanel()
	if not (loaded) then
		LibStub("AceConfig-3.0"):RegisterOptionsTable("ag_UnitFrames_2", blizzTbl)
		LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ag_UnitFrames_2", L["Options Frame"], L["ag_UnitFrames"])
		loaded = true
	end
	InterfaceOptionsFrame_OpenToFrame("Options Frame")
end

function options:UpdateSubgroups()
	for k, v in pairs(aUF.subgroups) do
		if not unitOptionsTable.args.RaidGroupOptions.args[k] then
			unitOptionsTable.args.RaidGroupOptions.args[k] = groupOptions
		end
	end
end

aUF.Options = options

