local L = LibStub("AceLocale-3.0"):GetLocale("ag_UnitFrames", true)
local media = LibStub("LibSharedMedia-3.0")
local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")

local newFrame, delFrame = aUF.newFrame, aUF.delFrame

local _G = getfenv(0)

local del, new = aUF.del, aUF.new

aUF.SetupStrings = {}

local defaultLayout = aUF.layouts.ABF

------------
-- SYSTEM --
------------

function aUF:NewClass(inherit)
	local c = {}
	c.prototype = {}
	if inherit then
		c.super = inherit
		setmetatable(c.prototype, {__index = c.super.prototype})
	end
	function c:new(...)
		local o = {}
		setmetatable(o, {__index = self.prototype})
		if o.init then
			o:init(...)
		end
		return o
	end
	return c
end

aUF.classes.aUFunit = aUF:NewClass()
aUF.classes.aUFunitPlayer = aUF:NewClass(aUF.classes.aUFunit)
aUF.classes.aUFunitPet = aUF:NewClass(aUF.classes.aUFunit)
aUF.classes.aUFunitCombo = aUF:NewClass(aUF.classes.aUFunit)
aUF.classes.aUFunitFocus = aUF:NewClass(aUF.classes.aUFunit)
aUF.classes.aUFunitMetro = aUF:NewClass(aUF.classes.aUFunit)

function aUF.classes.aUFunit:ToString()
	return "aUFunit"
end

local aUFunitClass = aUF.classes.aUFunit.prototype

function aUFunitClass:init(frame,unit,grouped,type)
	self.name = frame:GetName()
	self.frame = frame
	self.frame.object = self
	self.enabled = true
	
	self:SetupFrame()
	
    self.grouped = grouped
	if self.frame and unit then
		self:UnitReset(unit)
		self:UpdateAll()
		self.frame:Show()
		self.frame:SetAttribute("unit", unit)
	elseif type then
		self.grouped = true
		self:TypeReset(type)
	end
	self:SetAttributes()
end

function aUFunitClass:Reset(unitType)
	self:TypeReset(self.type)
	self:UnitReset(self.unit)
	self:UpdateAll()
end

function aUFunitClass:TypeReset(unitType)
	if not unitType then
		return
	end
	if self.type ~= unitType then
		self.type = unitType

		self.database = aUF.db.profile.units[self.type]
		
		self:ExecModuleMethods("OnTypeReset")

		if not InCombatLockdown() then
			self:ApplyLayout()
			self:BorderBackground()
			self:LoadScale()
			self:UpdateChildFrames()
			self:SetUnitWatch()
		end
		self.eventsRegistered = nil
	end
end

function aUFunitClass:UnitReset(unit)
	if not unit then return end
	if self.unit and self.unit == unit then return end
	
	self.unit = unit
	
	_, _, self.number = unit:find("(%d+)")
		
	local unitType = unit:gsub("%d", "")
	if not self.type or self.type ~= unitType then
		self:TypeReset(unitType)
	end

	if unit:find("pet") then
		if self.type == "partypet" or self.type == "raidpet" then
			self.parent = unit:gsub("pet","")
		else
			self.parent = "player"
		end
    end
end


function aUFunitClass:ObjectRegisterEvent(event,method)
	aUF:ObjectRegisterEvent(self, event, method)
end

function aUFunitClass:ObjectUnregisterEvent(event,method)
	aUF:ObjectUnregisterEvent(self, event, method)
end

function aUFunitClass:ExecModuleMethods(method)
	if aUF.modules then
		for k,v in pairs(aUF.modules) do
			if v:IsEnabled() and v[method] and (not v.inherit or self.type == v.inherit or v.inherit[self.type]) then
				v[method](v, self)
			end
		end
	end
end

function aUFunitClass:RegisterEvents()
	self:ObjectRegisterEvent("PARTY_MEMBERS_CHANGED", "RosterUpdate")
	self:ObjectRegisterEvent("RAID_ROSTER_UPDATE", "RosterUpdate")
	self:ObjectRegisterEvent("UNIT_HEALTH", "DisplayStatus")
end

function aUFunitClass:RosterUpdate()
	self:ExecModuleMethods("OnRosterUpdate")
end

-----------------
-- FRAME SETUP --
-----------------

local function onShow()
	local object = this.object
	local self = object
	if self then
		if self.Start then
			self:Start()
		end
		if self.frame.header then
			local unitName = UnitName(self.unit)
			if not self.lastName then
				self.lastName = unitName
			end
			if (self.lastName == unitName) then
				return
			end
		else
			self:UpdateAll()
		end
	end
end

local function onHide()
	local object = this.object
	if object then
		if object.Stop == "metro" then
			object:Stop()
		end	
		object:UpdateAll(true)
		this:StopMovingOrSizing()
	end
end

function aUFunitClass:AttributeChanged(name,value)
	if name ~= "unit" then return end
	local unit = aUF:UnitSuffix(value,self.frame:GetAttribute("unitsuffix"))
	if (unit) then
		aUF:RegisterUnit(self, unit)
	else
		aUF:UnregisterUnit(self)
	end	
	if self.frame then
		if unit then
			local unitname = UnitName(unit)
			if self.frame.header and (self.unit == unit and self.lastName == unitname) then
				return
			end
			self:UnitReset(unit)
			
			self.lastName = unitname
			self:UpdateAll()

			if self.childFrames then
				for k, v in pairs(self.childFrames) do
					v:AttributeChanged("unit",SecureButton_GetModifiedAttribute(v.frame,"unit"))
					v:UpdateAll()
				end
			end
		end
	end
end

local function onAttributeChanged(frame, name, value)
	local object = frame.object
	if object then
		object:AttributeChanged(name,value)
	end
end

local function OnShowMenu()
	this.object:DropDownUnit(this.object.type)
end

function aUFunitClass:SetupFrame()
	local top = newFrame("Frame", self.frame)
	top:SetFrameLevel(self.frame:GetFrameLevel()+3)
	self.top = top
	local middle = newFrame("Frame", self.frame)
	middle:SetFrameLevel(self.frame:GetFrameLevel()+2)
	self.middle = middle
	
	self.bars = {}
	self.strings = {}
	self.icons = {}
	
	self.frame:SetScript("OnAttributeChanged", onAttributeChanged)
	self.frame:SetScript("OnShow", onShow)
	self.frame:SetScript("OnHide", onHide)
	self.frame:SetScript("OnEnter", function() self:OnEnter() end)
	self.frame:SetScript("OnLeave", function() self:OnLeave() end)
	self.frame:SetScript("OnDragStart", function() self:OnDragStart(arg1) end)
	self.frame:SetScript("OnDragStop", function() self:OnDragStop(arg1) end)
	
	self.frame:SetMovable(true)
	self.frame:RegisterForDrag("LeftButton")
	self.frame:RegisterForClicks("LeftButtonUp","RightButtonUp","MiddleButtonUp","Button4Up","Button5Up")
	self.frame.custom_menu = OnShowCustomMenu
	self.frame.menu = OnShowMenu
end

function aUFunitClass:SetAttributes()
	self.frame:SetAttribute("*type1","target")
	self.frame:SetAttribute("*type2","custom_menu")
	self.frame:SetAttribute("type2","menu")
	self.frame:EnableMouse(true)

	ClickCastFrames = ClickCastFrames or {}
	ClickCastFrames[self.frame] = true
end

function aUFunitClass:LoadScale()	
	local themetable = aUF.layouts[self.database.FrameStyle or defaultLayout]
	if themetable and themetable.LoadScale then
		themetable:LoadScale(self)
	end
end

function aUFunitClass:BorderBackground()
	local themetable = aUF.layouts[self.database.FrameStyle or defaultLayout]
	if themetable and themetable.BorderBackground then
		themetable:BorderBackground(self)
	end
end

function aUFunitClass:LoadPosition()
	if self.grouped == true then return end
	self.frame:ClearAllPoints()
	if (aUF.db.profile.Positions[self.name]) then
		local x, y = strsplit(" ", aUF.db.profile.Positions[self.name])
		local scale = self.frame:GetEffectiveScale()
		self.frame:SetPoint("TOPLEFT", UIParent,"TOPLEFT", x/scale, y/scale)
	else
		self.frame:SetPoint("CENTER", UIParent, "CENTER")
	end
end

function aUFunitClass:SavePosition()
	local scale = self.frame:GetEffectiveScale()
	local worldscale = UIParent:GetEffectiveScale()

	local x,y = self.frame:GetLeft()*scale,self.frame:GetTop()*scale - (UIParent:GetTop())*worldscale
	
	aUF.db.profile.Positions[self.name] = strjoin(" ", x, y)
end

function aUFunitClass:ApplyLayout(reset)
	local themetable = aUF.layouts[self.database.FrameStyle or defaultLayout]
	
	if not aUF.SetupStrings[self.type] then
		aUF.SetupStrings[self.type] = {}
	end

	if reset and themetable.db then
		themetable.db:ResetProfile()
	end
	
    local height = self.database.Height
    self.frame:SetHeight(height)
	
	local frameWidth = self.database.Width    
	self.frame:SetWidth(frameWidth)
	
	self:ExecModuleMethods("OnParseLayout")
	
	themetable:ApplyLayout(self)
	
	self:ExecModuleMethods("OnSetWidth")
	self:ExecModuleMethods("OnSetHeight")    	
	self:ExecModuleMethods("OnSetBarTexture")
end

function aUFunitClass:UnapplyLayout()
	local themetable = aUF.layouts[self.database.FrameStyle or defaultLayout]
	themetable:UnapplyLayout(self)
end

function aUFunitClass:StatusBarsColor()
    self:ExecModuleMethods("OnStatusBarsColor")
end

function aUFunitClass:Enable()
    if self.frame.header then
		return
	end
	if not self.unit then
		return
	end
    self.enabled = true
	self:SetUnitWatch()
end

function aUFunitClass:Disable()
    if self.frame.header then
		return
	end
	if not self.unit then
		return
	end	
    self.enabled = false
	if not (InCombatLockdown()) then
		self:SetUnitWatch(true)
	end
	if self.childFrames then
		for k, v in pairs(self.childFrames) do
			v:Disable()
		end
	end
end

function aUFunitClass:SetUnitWatch(flag)
	if not (self.database.enabled) or flag then
		UnregisterUnitWatch(self.frame)
		self.frame:Hide()
	else
      RegisterUnitWatch(self.frame, false)
	end
end

local partyChildren = {"pet","target"}
local petPos = { "TOPRIGHT", "RIGHT", "BOTTOMRIGHT", "TOPLEFT", "LEFT", "BOTTOMLEFT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT", "TOPLEFT", "TOP", "TOPRIGHT"}
local parentPos = { "TOPLEFT", "LEFT", "BOTTOMLEFT", "TOPRIGHT", "RIGHT", "BOTTOMRIGHT", "TOPLEFT", "TOP", "TOPRIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"}
function aUFunitClass:UpdateChildFrames()
	for _, v in pairs(partyChildren) do
		local petType = self.type..v
		if aUF.db.profile.units[petType] and aUF.db.profile.units[petType].GroupWith == "parent" then
			if not self.childFrames then
				self.childFrames = {}
			end
			if not self.childFrames[v] then
				self:CreateChild(v)
			elseif self.childFrames and not (self.childFrames[v].enabled) then
				self.childFrames[v]:Enable()
			end
			
			local pos = aUF.db.profile.units[petType].PetPos
			self.childFrames[v].frame:ClearAllPoints()
			self.childFrames[v].frame:SetPoint(petPos[pos] or "LEFT",self.frame,parentPos[pos] or "RIGHT")
	
		elseif self.childFrames and self.childFrames[v] and self.childFrames[v].enabled then
			self.childFrames[v]:Disable()
		end
	end
end

function aUFunitClass:CreateChild(class)
	local kind
	if string.find(class, "target") then
		kind = "Metro"
	end
	
	local name = "aUF"..self.name..class
	local frame = CreateFrame("Button", name, parent or UIParent, "SecureUnitButtonTemplate")

	frame:SetParent(self.frame)		
	frame:SetAttribute("useparent-unit","true")
	frame:SetAttribute("unitsuffix",class)
	
	self.childFrames[class] = aUF.classes["aUFunit"..(kind or "")]:new(frame,nil,true,self.type..class)
	aUF.units[name] = self.childFrames[class]
end

-------------
-- UPDATES --
-------------

function aUFunitClass:UpdateAll(unload)
	if not self.database or not self.unit or not (self.enabled) then return end
	if not unload and self.database.enabled then
		self.exists = true
		self:ExecModuleMethods("OnUpdateAll")
		self:UpdateTextStrings()
		if not InCombatLockdown() then
			self:LoadPosition()
			self:UpdateChildFrames()
		end		
		if not (self.eventsRegistered) then
			if self.RegisterEvents then
				self:RegisterEvents()
			end
			self:ExecModuleMethods("OnRegisterEvents")
			self.eventsRegistered = true
		end
	else
		self.exists = false
		self.eventsRegistered = nil
		if self.pet then
			self.pet:Disable()
		end        
		self:ExecModuleMethods("OnDisableObject")
	end
end

function aUFunitClass:RegisterString(frame, tag, name)
	local layout = aUF.layouts[self.database.FrameStyle or defaultLayout]
	if not (frame) then return end
	if not aUF.SetupStrings[self.type] then
		aUF.SetupStrings[self.type] = {}
	end
	aUF.SetupStrings[self.type][name] = true
	
	if not (frame.updateFunc) then
		local func, events = aUF:FormatString(tag)
		if func then
			frame.updateFunc = function()
				if not self.unit then return end
				if frame:IsShown() and not frame.override then
					frame:SetText(func(self.unit))
				end
			end
		end
		if events then
			frame.events = events
			for j,i in pairs(frame.events) do
				self:ObjectRegisterEvent(i, frame.updateFunc)
			end
		end
	end
end

function aUFunitClass:UnregisterString(frame,name)
	if name and frame then
		aUF.SetupStrings[self.type][name] = nil
			if frame.events then
			for j,i in pairs(frame.events) do
				if frame.updateFunc then
					self:ObjectUnregisterEvent(i, frame.updateFunc)
				end
			end
		end
		frame.updateFunc = nil
		frame.events = nil	
		frame:SetText("")	
	end
end

function aUFunitClass:UpdateTextString(frame)
	if frame and frame:IsShown() then
		if frame.updateFunc and not frame.override then
			frame.updateFunc()
		end
	end
end

function aUFunitClass:UpdateTextStrings()
	self:DisplayStatus()
	for k,v in pairs(self.strings) do
		self:UpdateTextString(v)
	end
end

function aUFunitClass:DisplayStatus()
	local statusString = self.statusText
	if not (statusString) then
		return
	end
	
	local maxValue = UnitHealthMax(self.unit)
	if ( UnitIsDead(self.unit) ) then
		statusString:SetText(L["dead"])
		statusString.override = true
		return
	elseif ( UnitIsGhost(self.unit) ) then
		statusString:SetText(L["ghost"])
		statusString.override = true
		return
	elseif ( not UnitIsConnected(self.unit) ) then
		statusString:SetText(L["disc"])
		statusString.override = true
		return
	elseif ( maxValue == 1 ) then
		statusString:SetText(L["loading"])
		statusString.override = true
		return	
	elseif statusString.override == true then
		statusString.override = false
		statusString:SetText("")
		self:UpdateTextString(statusText)
	end
end

-----------------------
-- MOUSE INTERACTION --
-----------------------

function aUFunitClass:OnDragStart(button)
	if self.grouped then
        if self.frame:GetParent().object then
            self.frame:GetParent().object:OnDragStart(button)
        end
		return
    end
	if button == "LeftButton" and (aUF.db.profile.Locked == false or IsAltKeyDown()) then
		self.frame:StartMoving()
	end
end

function aUFunitClass:OnDragStop(button)
	if self.grouped then
        if self.frame:GetParent().object then
		self.frame:GetParent().object:OnDragStop(button)
        end
		return
	end
	self.frame:StopMovingOrSizing()
	self:SavePosition()
	self.frame:SetUserPlaced(false)
end



function aUFunitClass:OnEnter()
	self:ExecModuleMethods("OnEnter")
	if self.unit then
		self.frame.unit = self.unit
		UnitFrame_OnEnter(self.frame)
	end
end

function aUFunitClass:OnLeave()
	self:ExecModuleMethods("OnLeave")
	UnitFrame_OnLeave(self.frame)
end

function aUFunitClass:DropDownUnit()
	local type = nil

	if not self.number then
		self.number = 1
	end
	
	if self.unit == "player" then
		type = PlayerFrameDropDown
	elseif self.unit == "target" then
		type = TargetFrameDropDown
	elseif self.unit == "pet" then
		type = PetFrameDropDown
	elseif self.type == "party" then
		type = _G["PartyMemberFrame"..self.number.."DropDown"]
	elseif self.unit:find("raid") then
		type = FriendsDropDown
		this.unit = self.unit
		this.name = UnitName(self.unit)
		this.id = this:GetID()
		FriendsDropDown.displayMode = "MENU"
		FriendsDropDown.initialize = RaidFrameDropDown_Initialize
	end

	if self.number then
        this:SetID(self.number)
	end

	if type then
		HideDropDownMenu(1)
		type.unit = self.unit
		type.name = UnitName(self.unit)
		ToggleDropDownMenu(1, nil, type,"cursor")
		return true
	end
end

---------------------
-- TARGET SUBCLASS --
---------------------

function aUF.classes.aUFunitCombo.prototype:RegisterEvents()
	self:ObjectRegisterEvent("PLAYER_TARGET_CHANGED","TargetChanged")
	self:ObjectRegisterEvent("UNIT_HEALTH", "DisplayStatus")
end

function aUF.classes.aUFunitCombo.prototype:TargetChanged()
	if UnitExists(self.unit) and self.frame:IsVisible() then
		self:UpdateAll()
	end
end

---------------------
-- FOCUS SUBCLASS --
---------------------

function aUF.classes.aUFunitFocus.prototype:RegisterEvents()
	self:ObjectRegisterEvent("PLAYER_FOCUS_CHANGED","FocusChanged")
	self:ObjectRegisterEvent("UNIT_HEALTH", "DisplayStatus")
end

function aUF.classes.aUFunitFocus.prototype:FocusChanged()
	if self.exists then
		self:UpdateAll()
	end
end

-----------------------
-- ONUPDATE SUBCLASS --
-----------------------

aUF.classes.aUFunitMetro.prototype.RegisterEvents = false

function aUF.classes.aUFunitMetro.prototype:UpdateMetro(force)
	if self.unit and (force or self.frame:IsVisible()) then
		self:ExecModuleMethods("OnMetroUpdate")
		self:UpdateTextStrings()
		if self.unitName and self.unitName ~= UnitName(self.unit) then
			self:UpdateAll()
		end
	end
	self.unitName = self.unit and UnitName(self.unit)
end

function aUF.classes.aUFunitMetro.prototype:Start()
	if self.database.enabled then
		if not self.schedule then
			self.schedule = aUF.ScheduleRepeatingTimer(self, "UpdateMetro", 0.5)
		end
		self:UpdateMetro()
	end
end

function aUF.classes.aUFunitMetro.prototype:Stop()
	if self.schedule then
		aUF.CancelTimer(self, self.schedule)
		self.schedule = nil
	end
	self:UpdateMetro()
end
