local _G = getfenv(0)

local aUF = LibStub("AceAddon-3.0"):NewAddon("ag_UnitFrames", "AceTimer-3.0")

local AceDB = LibStub("AceDB-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("ag_UnitFrames", true)
local SM = LibStub("LibSharedMedia-3.0")

-----------------------------------------------------------------------
-- Local heap and utility functions
-----------------------------------------------------------------------

local new, del
do
	local cache = setmetatable({},{__mode="k"})
	function new(...)
		local t = next(cache)
		if t then
			cache[t] = nil
			for i = 1, select("#", ...) do
				table.insert(t, (select(i, ...)))
			end
			return t
		else
			return {...}
		end
	end
	function del(t, notDeep)
		if type(t) == "table" then
			for k, v in pairs(t) do
				if not (notDeep) and type(v) == "table" then
					del(v)
				end
				t[k] = nil
			end
			cache[t] = true
	end
		return nil
	end
end

aUF.del = del
aUF.new = new

local function tblCopy(src, dest)
	for k,v in pairs(src) do
		if type(v) == "table" then
			if not (type(dest[k]) == "table") then
				dest[k] = {}
			end
			tblCopy(v, dest[k])
		else
			dest[k] = v
		end
	end
end

local newFrame, delFrame
do
	local frameCache = {}
	local frameCount = {}
	function newFrame(kind, parent, extra)
		local frame
		if frameCache[kind] then
			frame = table.remove(frameCache[kind])
			if #frameCache[kind] == 0 then
				frameCache[kind] = del(frameCache[kind])
			end
			frame:SetParent(parent)
			if kind == "Texture" or kind == "FontString" then
				frame:SetDrawLayer(extra)
			end
			frame:Show()
		else
			frameCount[kind] = (frameCount[kind] or 0) + 1
			local name = "aUF" .. kind .. frameCount[kind]
			if kind == "Texture" then
				frame = parent:CreateTexture(name, extra)
			elseif kind == "FontString" then
				frame = parent:CreateFontString(name, extra)
			else
				if kind == "Cooldown" then
					frame = CreateFrame("Cooldown", name, parent, "CooldownFrameTemplate")
					frame:Show()
				else
					frame = CreateFrame(kind, name, parent)
				end
			end
		end
		return frame
	end

	function delFrame(frame)
		if not frame then return nil end
		local kind = frame:GetObjectType()
		if kind == "FontString" then
			frame:SetText("")
			frame:SetJustifyH("CENTER")
			frame:SetJustifyV("MIDDLE")
			frame:SetNonSpaceWrap(true)
			frame:SetTextColor(1, 1, 1, 1)
			frame:SetFontObject(nil)
		elseif kind == "Texture" then
			frame:SetTexture(nil)
			frame:SetVertexColor(1, 1, 1, 1)
			frame:SetBlendMode("BLEND")
			frame:SetDesaturated(false)
			frame:SetTexCoord(0, 1, 0, 1)
			frame:SetTexCoordModifiesRect(false)
		elseif kind == "StatusBar" then
			frame:SetStatusBarColor(1, 1, 1, 1)
			frame:SetStatusBarTexture(nil)
			frame:SetValue(1)
			frame:SetOrientation("HORIZONTAL")
		end
		frame:Hide()
		if frame.SetBackdrop then
			frame:SetBackdrop(nil)
		end
		frame:SetParent(UIParent)
		frame:ClearAllPoints()
		frame:SetAlpha(1)
		frame:SetHeight(0)
		frame:SetWidth(0)
		local frameCache_kind = frameCache[kind]
		if not frameCache_kind then
			frameCache_kind = new()
			frameCache[kind] = frameCache_kind
		end
		frameCache_kind[#frameCache_kind+1] = frame
		return nil
	end
end

aUF.delFrame = delFrame
aUF.newFrame = newFrame

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

aUF.RepColor = {
	[1] = {r = 226/255, g = 45/255, b = 75/255},
	[2] = {r = 226/255, g = 45/255, b = 75/255},
	[3] = {r = 0.75, g = 0.27, b = 0},
	[4] = {r = 1, g = 1, b = 34/255},
	[5] = {r = 0.2, g = 0.8, b = 0.15},
	[6] = {r = 0.2, g = 0.8, b = 0.15},
	[7] = {r = 0.2, g = 0.8, b = 0.15},
	[8] = {r = 0.2, g = 0.8, b = 0.15},
}

aUF.TapDeadColor = {
	r = 0.5, g = 0.5, b = 0.5
}

local defaultVars = {
	profile = {
		Locked = false,
		BarStyle = "Aluminium",
		BarFadeTime = 0.5,
		CurrentRaidSet = "1",
		HidePartyInRaid = true,
		FiveMan = true,
		
		CastbarColor = {
			r = 1.0,
			g = 0.7,
			b = 0.0,
		},
		HealthColor = {
			r = 0.41,
			g = 0.95,
			b = 0.2,
		},
		ManaColor = {
			[0] = { b = 0.84, g = 0.52, r = 0.28},
			[1] = { r = 226/255, g = 45/255, b = 75/255},
			[2] = { r = 255/255, g = 210/255, b = 0},
			[3] = { r = 255/255, g = 220/255, b = 25/255},
			[4] = { r = 0.00, g = 1.00, b = 1.00}
		},
		TapDeadColor = {
			r = 0.5,
			g = 0.5,
			b = 0.5,
		},
		PartyFrameColors = {
			r = 0,
			g = 0,
			b = 0,
			a = 0.5
		},
		FrameBorderColors = {
			a = 1,
			b = 0.47,
			g = 0.28,
			r = 0.31,
		},
		XPColor = {
			r = 0.8,
			g = 0,
			b = 0.7,
		},
		RestedXPColor = {
			r = 0,
			g = 0,
			b = 1,
		},
		units = {
			["**"] = {
				BorderStyle = "SquareClean",
				fontOverride = false,
				fontOverrideSize = 0,
				fontOverrideType = "Myriad Condensed Web",
				FrameStyle = "ABF",
				Scale = 1,
				Width = 190,
				Height = 66,
				enabled = false,
			},
			player = {
				enabled = true,
				ShowXP = true,
			},
			pet = {
				enabled = true,
				ShowXP = true,
				Height = 60,
			},
			party = {
				enabled = true,
				GroupWith = "self",
				enabled = true,
				Height = 60,
				HideInRaid = true,
			},
			raid = {
				enabled = false,
				Scale = 1,
				Width = 110,
				Height = 28,
			},
			raidpet = {
				Scale = 1,
				Width = 110,
				Height = 28,
			},
			target = {
				enabled = true,
				TargetShowHostile = true,
			},
			targettarget = {
				enabled = true,
				Scale = 1,
				Width = 135,
				Height = 28,
			},
			pettarget = {
				Scale = 1,
				Width = 135,
				Height = 28,
			},
			partytarget = {
				GroupWith = "parent",
				Scale = 1,
				Width = 100,
				Height = 25,
				PetPos = 4,
			},
			partypet = {
				enabled = true,
				GroupWith = "parent",
				Scale = 1,
				Width = 100,
				Height = 25,
				PetPos = 6,
			},
			targettargettarget = {
				Scale = 1,
				Width = 135,
				Height = 28,
			},
		},
		partygroup = {
				Grow = "TOP",
				ShowAnchor = false,
				Exists = true,
				groupFilter = "",
				Padding = 21,
				AnchorOffset = 2,
		},
		partypetgroup = {
				Grow = "TOP",
				ShowAnchor = true,
				Exists = true,
				groupFilter = "",
				Padding = 21,
				AnchorOffset = 2,
				suffix = "pet",
		},
		subgroups = {
			["**"] = {
				Name = "New Group",
				Grow = "TOP",
				ShowAnchor = true,
				Padding = 2,
				AnchorOffset = 2,
				RaidSets = {"1","2","3","4","5","6","7","8","9","10"},
				groupFilter = "",
				groupBy = "GROUP",
			},
			group1 = {
				Name = "Group 1",
				groupFilter = "1",
				Exists = true,
			},
			group2 = {
				Name = "Group 2",
				groupFilter = "2",
				Exists = true,
			},
			group3 = {
				Name = "Group 3",
				groupFilter = "3",
				Exists = true,
			},
			group4 = {
				Name = "Group 4",
				groupFilter = "4",
				Exists = true,
			},
			group5 = {
				Name = "Group 5",
				groupFilter = "5",
				Exists = true,
			},
			group6 = {
				Name = "Group 6",
				groupFilter = "6",
				Exists = true,
			},
			group7 = {
				Name = "Group 7",
				groupFilter = "7",
				Exists = true,
			},
			group8 = {
				Name = "Group 8",
				groupFilter = "8",
				Exists = true,
			},
		},
		Positions = {
			aUFplayer = "10 -13",
			aUFpet = "10 -90",
			aUFtarget = "250 -13",
			aUFtargettarget = "450 -13",
			aUFtargettargettarget = "450 -30",
			party = "10 -160",
			partypet = "185 -109",
		},
		modulesDisabled = {
			ag_UnitFrames_Pvpicon = true,
			ag_UnitFrames_Rangecheck = true,
		},
	},
}

aUF.Borders = {
	RoundBlizz = {texture = "Interface\\Tooltips\\UI-Tooltip-Border",size = 16,insets = 4},
	SquareBlizz  = {texture = "Interface\\DialogFrame\\UI-DialogBox-Border",size = 16,insets = 5},
	SquareClean  = {texture = "Interface\\AddOns\\ag_UnitFrames\\images\\UI-Square-Border.tga",size = 8,insets = 4},
	Hidden  = {texture = "",size = 0,insets = 3, tileSize= 1},
}

local bars = {
	Otravi  	 = "Interface\\AddOns\\ag_UnitFrames\\images\\bars\\AceBarFrames.tga",
	Smooth 		 = "Interface\\AddOns\\ag_UnitFrames\\images\\bars\\smooth.tga",
	Gloss  		 = "Interface\\AddOns\\ag_UnitFrames\\images\\bars\\Gloss.tga",
	BantoBar	 = "Interface\\AddOns\\ag_UnitFrames\\images\\bars\\BantoBar.tga",
	Aluminium    = "Interface\\AddOns\\ag_UnitFrames\\images\\bars\\Aluminium.tga",
}

------------
-- SYSTEM --
------------

local events = {}
aUF.layouts = {}
aUF.classes = {}
aUF.units = {}
aUF.subgroups = {}

local function optionsfunc()
	local loaded = LoadAddOn("ag_Options")
	if loaded or aUF.Options then
		aUF.Options:CreateBlizzPanel()
	else
		aUF:Print(L["ag_Options addon isn't available."])
	end
end

local function showoptionsgui()
	local loaded = LoadAddOn("ag_Options")
	if loaded or aUF.Options then
		aUF.Options:OpenOptions()
	else
		aUF:Print(L["ag_Options addon isn't available."])
	end
end

function aUF:Print(text)
	DEFAULT_CHAT_FRAME:AddMessage(L["ag_UnitFrames"]..": "..(text or "nil"))
end

function aUF:OnInitialize()
	for n,t in pairs(bars) do
		SM:Register("statusbar", n, t)
	end
	SM:Register("font", "Myriad Condensed Web", "Interface\\AddOns\\ag_UnitFrames\\fonts\\barframes.ttf")

	self.db = LibStub("AceDB-3.0"):New("aUF2DB", defaultVars, "global")
	
	self.eventFrame = CreateFrame("Frame")
end

local function updateMedia()
	aUF:CallUnitMethods("ExecModuleMethods","OnSetBarTexture")
end

function aUF:OnEnable()
	local frame = CreateFrame("FRAME", nil)
	frame:SetScript("OnShow",function() optionsfunc() end)
	frame.name = L["ag_UnitFrames"]
	InterfaceOptions_AddCategory(frame)
	InterfaceOptionsFrame:SetMovable(true)
	
	local title = InterfaceOptionsFrame:CreateTitleRegion()
	title:ClearAllPoints()
	title:SetAllPoints(InterfaceOptionsFrameHeader)
	self.frame = frame

	_G["SLASH_AGUNITFRAMES1"] = "/aguf"
	SlashCmdList["AGUNITFRAMES"] = showoptionsgui

	self:RegisterEvents()
	aUF:LoadUnits()
	aUF:RAID_ROSTER_UPDATE(true)
	SM.RegisterCallback(aUF, "LibSharedMedia_Registered", updateMedia)
	SM.RegisterCallback(aUF, "LibSharedMedia_SetGlobal", updateMedia)
end

aUF.soloUnits = {player = true,pet = true,target = true,targettarget = true,targettargettarget = true,focus = true,focustarget = true,pettarget = true}
aUF.partyUnits = {partytarget = true,party = true,partypet = true}
aUF.raidUnits = {raid = true}

function aUF:LoadUnits()
	for k in pairs(aUF.soloUnits) do
		aUF:LoadUnit(k)
	end
	for k in pairs(aUF.partyUnits) do
		aUF:LoadUnit(k)
	end
	for k in pairs(aUF.raidUnits) do
		aUF:LoadUnit(k)
	end	
end

function aUF:LoadUnit(class)
	if aUF.soloUnits[class] then
		aUF:LoadSoloUnits(class)
	elseif aUF.partyUnits[class] then
		aUF:LoadPartyUnits(class)
	elseif aUF.raidUnits[class] then	
		aUF:LoadRaidUnits(class)
	end
end

function aUF:LoadSoloUnits(class)
	if not (self.db.profile.units[class].enabled) then return end
	if class == "player" then
		aUF:CreateUnit(class,"Player")
	elseif class == "pet" then
		aUF:CreateUnit(class,"Pet")
	elseif class == "target" then
		aUF:CreateUnit(class,"Combo")
	elseif class == "focus" then
		aUF:CreateUnit(class,"Focus")			
	else
		if string.find(class, ".+target") then
			aUF:CreateUnit(class,"Metro")
		else
			aUF:CreateUnit(class)
		end
	end
	if self.Options and self.Options.loaded then
		aUF.Options:AddUnitOptions(class)
		aUF.Options:UpdateStringOptions(class)
	end
end

function aUF:LoadPartyUnits(class)
	if not (self.db.profile.units[class].enabled) then return end
	aUF:UpdatePartyGrouping(class)
	if self.Options and self.Options.loaded then
		aUF.Options:AddUnitOptions(class)
		aUF.Options:UpdateStringOptions(class)
	end
end

function aUF:LoadRaidUnits(class)
	if not (self.db.profile.units[class].enabled) then return end
	for k,v in pairs(self.db.profile.subgroups) do
		if v.Exists == true and not aUF.subgroups[k] then
			aUF:CreateSubgroup(k,class)
		end
	end
	if self.Options and self.Options.loaded then
		aUF.Options:AddUnitOptions(class)
		aUF.Options:UpdateStringOptions(class)
	end
end

function aUF:DisableAllFrames(class)
	if not class then return end
	for k, v in pairs(self.units) do
		if v.type == class and not (v.frame.header) then
			v:Disable()
		end
	end
	for k,v in pairs(self.subgroups) do
		if v.type == class then
			v:Disable()
		end
	end
end

function aUF:EnableAllFrames(class)
	if not class then return end
	for k, v in pairs(self.units) do
		if v.type == class and not (v.frame.header) then
			v:Enable()
		end
	end
	for k,v in pairs(self.subgroups) do
		if v.type == class then
			v:Enable()
		end
	end
end

--------------------------------------
-- OBJECT CREATION/METHOD EXECUTION --
--------------------------------------

function aUF:CreateUnit(unit,class,name,grouped)
	local name = "aUF"..(name or unit)
	if not _G[name] then
		local frame = CreateFrame("Button", name, parent or UIParent, "SecureUnitButtonTemplate")
		frame:Hide()
		self.units[name] = self.classes["aUFunit"..(class or "")]:new(frame,unit,grouped)
	end
	return self.units[name]
end

function aUF:CreateSubgroup(name,type,reset)
	if not aUF.subgroups[name] then
		aUF.subgroups[name] = aUF.classes.aUFgroup:new(name,type,reset)
	else
		aUF.subgroups[name]:SetDefaults(true)
		aUF.subgroups[name].database.Exists = true
		aUF:Print(name)
		aUF.subgroups[name]:Enable()
	end
	return aUF.subgroups[name]
end

function aUF:CreateNewGroup()
	local i = 1
	local k = "group"..i
	while (self.db.profile.subgroups[k] and self.db.profile.subgroups[k].Exists == true) do
		i = i + 1
		k = "group"..i
	end
	aUF:CreateSubgroup(k,"raid",true)

	return k
end

function aUF:CallUnitMethods(func,arg,find,type,object)
	if not func then return end
	if not object then object = "units" end
	if self[object] then
		for _,unitObject in pairs(self[object]) do
			if (unitObject[type] == find or not (type)) and unitObject[func] and unitObject.unit then
				unitObject[func](unitObject,arg)
			end
		end
	end
end

-----------
-- Units --
-----------

function aUF:RegisterUnit(object, unit)
	if not self.unitid then
		self.unitid = {}
	end

	if (not self.unitid[unit]) then
		self.unitid[unit] = {}
	end

	aUF:UnregisterUnit(object)

	self.unitid[unit][object] = true
end

function aUF:UnregisterUnit(object)
	if type(self.unitid) ~= "table" then return end
	for id,objs in pairs(self.unitid) do
		if (objs[object]) then
			objs[object] = nil
		end
	end
end

-----------------------
-- EVENT REGISTERING --
-----------------------
local coreEvents = {
	RAID_ROSTER_UPDATE = true,
	PLAYER_REGEN_ENABLED = true,
	PLAYER_LEAVING_WORLD = true,
	PLAYER_ENTERING_WORLD = false,
}

function aUF:RegisterEvents()
	self.eventFrame:SetScript("OnEvent", aUF.OnEvent)

	self.eventFrame:RegisterEvent("RAID_ROSTER_UPDATE")
	self.eventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
	self.eventFrame:RegisterEvent("PLAYER_LEAVING_WORLD")
	self.eventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
	self.eventFrame:RegisterEvent("PLAYER_LOGIN")
end

function aUF:OnEvent(event, unit, ...)
--	aUF:Print(hat)
	
	if coreEvents[event] then
		aUF[event](self)
	end
	if unit and aUF.unitid and aUF.unitid[unit] then
		for object in pairs(aUF.unitid[unit]) do
			if events[object] and events[object][event] then
				for method in pairs(events[object][event]) do
					if type(method) == "function" then

					
						method(object, event, unit, ...)
					else


						object[method](object, event, unit, ...)
					end
				end
			end
		end
	else
		for object, methods in pairs(events) do
			if methods[event] then
				for method in pairs(methods[event]) do
					if object[method] then
						object[method](object, event, ...)
					elseif type(method) == "function" then
						method(object, event, ...)
					end
				end
			end
		end
	end
end

function aUF:ObjectRegisterEvent(object,event,method)
	if not events[object] then
		events[object] = new()
	end
	if not events[object][event] then
		events[object][event] = new()
	end
	events[object][event][method] = true

	self.eventFrame:RegisterEvent(event)
end

function aUF:ObjectUnregisterEvent(object,event,method)
	if events[object] then
		if events[object][event] then
			if events[object][event][method] then
				events[object][event][method] = del(events[object][event][method])
			end
			if next(events[object][event]) == nil then
				events[object][event] = del(events[object][event])
			end
		end
	end
	if not (coreEvents[event] == nil) then
		return
	end
	for object, objEvents in pairs(events) do
		if objEvents[event] then
			return
		end
	end
	self.eventFrame:UnregisterEvent(event)
end

function aUF:ObjectUnregisterAllEvents(object)
	if not events[object] then return end
	for event in pairs(events[object]) do
		self:ObjectUnregisterEvent(object,event,method)
	end
end

---------------------
-- UNIT VISIBILITY --
---------------------

function aUF:UpdatePartyGrouping(class)
	local groupWith = aUF.db.profile.units[class] and aUF.db.profile.units[class].GroupWith
	aUF:DisableAllFrames(class)
	if groupWith == "parent" then
		for k,v in pairs(self.units) do
			if v.type == "party" then
				v:UpdateChildFrames()
			end
		end
	elseif groupWith == "self" then
		if self.subgroups[class] then
			self.subgroups[class]:Enable()
		else
			aUF:CreateSubgroup(class,class)
		end
	elseif groupWith == "free" then
		for i=1,4 do
			local unit = class..i
			if self.units["aUF"..unit] then
				self.units["aUF"..unit]:Enable()
			else
				if string.find(class, "target") then
					aUF:CreateUnit(unit,"Metro")
				else
					aUF:CreateUnit(unit)
				end
			end
		end
	end
end

local raidDirty
function aUF:RAID_ROSTER_UPDATE(force)	
	local inRaid = self.inRaid
	if (aUF.db.profile.FiveMan and GetNumRaidMembers()>5) or (not(aUF.db.profile.FiveMan) and GetNumRaidMembers()>0) then
		self.inRaid = true
	else
		self.inRaid = false
	end
	
	local raidSet = aUF.db.profile.CurrentRaidSet
	if (inRaid ~= self.inRaid) or force then
		if InCombatLockdown() then
			raidDirty = true
		else
			if aUF.db.profile.units.raid and aUF.db.profile.units.raid.enabled then
				for k,v in pairs(aUF.subgroups) do
					if v.type == "raid" then
						if self.inRaid and (v.database.RaidSets and v.database.RaidSets[tonumber(raidSet)]) then
							v:Enable()
						else
							v:Disable()
						end
					end
				end
			end
			if self.inRaid and aUF.db.profile.HidePartyInRaid then
				for k in pairs(aUF.partyUnits) do
					aUF:DisableAllFrames(k)
				end
			elseif aUF.db.profile.units.party and aUF.db.profile.units.party.enabled then
				for k in pairs(aUF.partyUnits) do
					if aUF.db.profile.units[k] and aUF.db.profile.units[k].enabled then
						aUF:UpdatePartyGrouping(k)
					end
				end			
			end
		end
	end
end

function aUF:PLAYER_REGEN_ENABLED()
	if raidDirty and self.subgroups then
		for k,v in pairs(self.subgroups) do
			v:SetVisibility()
		end
		raidDirty = nil
	end
end

function aUF:PLAYER_ENTERING_WORLD()
	aUF:ShowRaidHeaders()
	coreEvents["PLAYER_ENTERING_WORLD"] = false
	coreEvents["PLAYER_LEAVING_WORLD"] = true
end

function aUF:ShowRaidHeaders()
	for k,v in pairs(aUF.subgroups) do
		v.forcedHidden = false
	end
	aUF:RAID_ROSTER_UPDATE(true)
end

function aUF:PLAYER_LEAVING_WORLD()
	for k,v in pairs(aUF.subgroups) do
		v.forcedHidden = true
		v.header:Hide()
		v.anchor:Hide()
	end
	coreEvents["PLAYER_LEAVING_WORLD"] = false
	coreEvents["PLAYER_ENTERING_WORLD"] = true
end

function aUF:SetRaidset(option)
	set = tonumber(self.db.profile.CurrentRaidSet)
	if not (InCombatLockdown()) and set then
		if option == "prev" then
			if set <= 1 then
				self.db.profile.CurrentRaidSet = "10"
			else
				self.db.profile.CurrentRaidSet = tostring(set - 1)
			end
		elseif option == "next" then
			if tonumber(set) >= 10 then
				self.db.profile.CurrentRaidSet = "1"
			else
				self.db.profile.CurrentRaidSet = tostring(set + 1)
			end
		else
			self.db.profile.CurrentRaidSet = tostring(option)
		end
		aUF:RAID_ROSTER_UPDATE(true)
		self:Print(string.format("Raidset %s activated.",self.db.profile.CurrentRaidSet))
	elseif set then
		self:Print("You cannot change the raid set while in combat.")
	end
end

-------------
-- MODULES --
-------------

local modPrototype = {}

function modPrototype:OnInitialize()
	if self.OnRegister then
		self:OnRegister()
	end
	if aUF.db.profile.modulesDisabled[self.name] then
		self.enabledState = false
	end
end

function modPrototype:RegisterDefaults(defaults)
	tblCopy(defaults, defaultVars.profile)
	-- we gotta do this to make sure the new defaults are registered
	aUF.db = LibStub("AceDB-3.0"):New("aUF2DB", defaultVars)
end

modPrototype.ScheduleRepeatingTimer = aUF.ScheduleRepeatingTimer
modPrototype.CancelTimer = aUF.CancelTimer

aUF:SetDefaultModulePrototype(modPrototype)

---------------
-- UTILITIES --
---------------

function aUF:UnitSuffix(unit,suffix)
	if not suffix then return unit end
	if unit then
		unit = unit..suffix
		if unit:find("pet") then
			unit = gsub(unit, "^([^%d]+)([%d]+)[pP][eE][tT]","%1pet%2")
		end
		return unit
	end
end
		
function aUF:UtilFactionColors(unit)
	local r, g, b = 0,0,0
	local a = 0.5
	if ( UnitPlayerControlled(unit) ) then
		if ( UnitCanAttack(unit, "player") ) then
			if ( not UnitCanAttack("player", unit) ) then
				r,g,b = 0.84, 0.52, 0.28
			else
				r,g,b = 0.88, 0.17, 0.29
			end
		elseif ( UnitCanAttack("player", unit) ) then
			r,g,b = 1, 0.97, 0.1
		elseif ( UnitIsPVP(unit) ) then
			r,g,b = 0.41, 0.95, 0.2
		else
			r,g,b = 0.84, 0.52, 0.28
		end
	elseif ( UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) ) or UnitIsDead(unit) then
		return self.TapDeadColor
	else
		local reaction = UnitReaction(unit, "player")
		if ( reaction ) then
			if reaction == 5 or reaction == 6 or reaction == 7 then
				r,g,b = 0.41, 0.95, 0.2
			elseif reaction == 4 then
				r,g,b = 1, 0.97, 0.1
			elseif reaction == 1 or reaction == 2 or reaction == 3 then
				r,g,b = 0.88, 0.17, 0.29
			else
				return UnitReactionColor[reaction]
			end
		end
	end
	return {r = r,g = g,b = b}
end

function aUF:GiveHex(r,g,b)
	local r = math.min(r*255,255)
	local g = math.min(g*255,255)
	local b = math.min(b*255,255)
	return string.format("|cff%2x%2x%2x", r, g, b) or ""
end

function aUF:GetRaidColors(class)
	if RAID_CLASS_COLORS[class] then
		local tbl = RAID_CLASS_COLORS[class]
		return string.format("|cff%2x%2x%2x", min(tbl.r*255,255), min(tbl.g*255,255), min(tbl.b*255,255)) or ""
	else
		return "|r"
	end
end

----
----

local function formatLargeValue(value)
	if value < 9999 then
		return value
	elseif value < 999999 then
		return string.format("%.1fk", value / 1000)
	else
		return string.format("%.2fm", value / 1000000)
	end
end

aUF.formats = {}

aUF.textFunctions = {
	["curhp"] = {
		func = function (u)
			if UnitInParty(u) or UnitInRaid(u) or u == "player" or u == "pet" then
				return formatLargeValue(UnitHealth(u)).."/"..formatLargeValue(UnitHealthMax(u))
			elseif MobHealth3 then
				local currValue,maxValue,MHfound = MobHealth3:GetUnitHealth(u, UnitHealth(u), UnitHealthMax(u))
				if MHfound then
					return formatLargeValue(currValue).."/"..formatLargeValue(maxValue)
				else
					return nil
				end
			else
				return nil
			end
		end,
		order = 44,
		events = "UNIT_HEALTH",
		name = L["Absolute HP"],
		desc = L["Current and maximum HP in absolute values"],
	},
	["curmp"] = {
		func = function (u)
			local mpmax = UnitManaMax(u)
			if mpmax > 1 then
				return formatLargeValue(UnitMana(u)).."/"..formatLargeValue(mpmax)
			end
		end,
		order = 43,
		events = {"UNIT_MANA", "UNIT_RAGE", "UNIT_FOCUS", "UNIT_ENERGY", "UNIT_DISPLAYPOWER" },
		name = L["Absolute Mana"],
		desc = L["Current and maximum MP in absolute values"],
	},
	["percenthp"] =  {
		func = function (u)
			local hpmax = UnitHealthMax(u)
			return (hpmax ~= 0) and (floor((UnitHealth(u) / hpmax) * 100)).."%" or "0%"
		end,
		order = 42,
		events = "UNIT_HEALTH",
		name = L["Percent HP"],
		desc = L["Current HP in percent (Unknown value for people not in party/raid)"],
	},
	["percentmp"] =  {
		func = function (u)
			local mpmax = UnitManaMax(u)
			return (mpmax > 1) and (floor((UnitMana(u) / mpmax) * 100)).."%" or "0%"
		end,
		order = 41,
		events = {"UNIT_MANA", "UNIT_RAGE", "UNIT_FOCUS", "UNIT_ENERGY", "UNIT_DISPLAYPOWER" },
		name = L["Percent MP"],
		desc = L["Current MP in percent"],
	},
	["diffhp"] = {
		func = function (u)
			if UnitInParty(u) or UnitInRaid(u) or u == "player" or u == "pet" then
				local v = math.abs(UnitHealth(u) - UnitHealthMax(u))
				if v > 0 then
					return "-"..formatLargeValue(v)
				end
			elseif MobHealth3 then
				local currValue,maxValue,MHfound = MobHealth3:GetUnitHealth(u, UnitHealth(u), UnitHealthMax(u))
				if MHfound then
					local v = math.abs(currValue - maxValue)
					if v > 0 then
						return "-"..formatLargeValue(v)
					end
				end
			end
			return nil
		end,
		order = 40,
		events = "UNIT_HEALTH",
		name = L["Difference HP"],
		desc = L["Current HP minus maximum HP"],
	},
	["diffmp"] = {
		func = function (u)
			local v = math.abs(UnitMana(u) - UnitManaMax(u))
			if v > 0 then
				return "-"..formatLargeValue(v)
			end
		end,
		order = 39,
		events = {"UNIT_MANA", "UNIT_RAGE", "UNIT_FOCUS", "UNIT_ENERGY", "UNIT_DISPLAYPOWER" },
		name = L["Difference MP"],
		desc = L["Current MP minus maximum MP"],
	},
	["name"] = {
		func = function(u)
			local type = u:gsub("%d", "")
			if aUF.db.profile.units[type].RaidColorName and UnitIsPlayer(u) then
				local _,x=UnitClass(u)
				return string.format("%s%s%s",aUF:GetRaidColors(x) or "",UnitName(u) or "","|cFFFFFFFF")
			else
				return UnitName(u) or ""
			end
		end,
		order = 11,
		events = {"UNIT_NAME_UPDATE", "PARTY_MEMBERS_CHANGED"},
		name = L["Name"],
		desc = L["Character Name"],
	},
	["mobtype"] = {
		func = function(u)
			if UnitClassification(u) == "rare" then
				return L["Rare"]
			elseif UnitClassification(u) == "rareelite" then
				return L["Rare-Elite"]
			elseif UnitClassification(u) == "elite" then
				return L["Elite"]
			elseif UnitClassification(u) == "worldboss" then
				return L["Boss"]
			else
				return nil
			end
		end,
		order = 21,
		name = L["Mobtype"],
		desc = L["Elite, boss or rare"],
	},
	["level"] = {
		func = function (u)
			local x = UnitLevel(u)
			return ((x>0) and x or "??")
		end,
		order = 22,
		events = {"UNIT_NAME_UPDATE", "PARTY_MEMBERS_CHANGED","UNIT_LEVEL"},
		name = L["Level"],
		desc = L["Character level"],
	},
	["class"] = {
		func = function (u)
			if UnitIsPlayer(u) then
				local _,x=UnitClass(u)
				return string.format("%s%s%s",aUF:GetRaidColors(x) or "",(UnitClass(u) or L["Unknown"]) or "","|cFFFFFFFF")
			else
				return	(UnitCreatureFamily(u) or UnitCreatureType(u) or "")
			end
		end,
		order = 23,
		name = L["Class"],
		desc = L["Character Class"],
		events = {"UNIT_NAME_UPDATE", "PARTY_MEMBERS_CHANGED"},
	},
	["race"] = {
		func = function (u)
			return UnitRace(u) or ""
		end,
		order = 24,
		name = L["Race"],
		desc = L["Character Race"],
		events = {"UNIT_NAME_UPDATE", "PARTY_MEMBERS_CHANGED"},
	},
	["raidgroup"] = {
		func = function (u)
			for i=1, GetNumRaidMembers() do
				local name, rank, subgroup = GetRaidRosterInfo(i)
				if (name == UnitName(u)) then
					return "(" .. subgroup .. ")"
				end
			end
			return ""
		end,
		events = "RAID_ROSTER_UPDATE",
		order = 51,
		name = L["Raidgroup"],
		desc = L["Current raid subgroup"],
	},
	["afkdnd"] = {
		func = function(u)
			if UnitIsAFK(u) then
				return "(AFK)"
			elseif UnitIsDND(u) then
				return "(DND)"
			else
				return nil
			end
		end,
		events = "PLAYER_FLAGS_CHANGED",
		order = 52,
		name = L["AFK/DND Status"],
		desc = L["Displays the AFK or DND status"],
	},
	["curxp"] = {
		func = function(u)
			local curXP, totalXP
			if u == 'pet' then
				curXP, totalXP = GetPetExperience()
			else
				curXP, totalXP = UnitXP(u), UnitXPMax(u)
			end
			return curXP and (curXP .. '/' .. totalXP) or ''
		end,
		events = {"PLAYER_XP_UPDATE", "UNIT_PET_EXPERIENCE"},
		order = 61,
		name = L["Current XP"],
		desc = L["Current and total required XP"],
	},
	["percentxp"] = {
		func = function(u)
			local curXP, totalXP
			if u == 'pet' then
				curXP, totalXP = GetPetExperience()
			else
				curXP, totalXP = UnitXP(u), UnitXPMax(u)
			end
			return (curXP and (totalXP > 0)) and (floor((curXP / totalXP) * 100).."%") or "0%"
		end,
		events = {"PLAYER_XP_UPDATE", "UNIT_PET_EXPERIENCE"},
		order = 62,
		name = L["Percent XP"],
		desc = L["Current XP in percent"],
	},
	["restxp"] = {
		func = function(u)
			if u == 'player' then
				return GetXPExhaustion() or 0
			end
		end,
		events = "UPDATE_EXHAUSTION",
		order = 63,
		name = L["Rested XP"],
		desc = L["Rested XP"],
	},
	["percentrestxp"] = {
		func = function(u)
			if u == 'player' then
				local totalXP, restXP = UnitXPMax(u), GetXPExhaustion() or 0
				return (totalXP > 0) and (floor((restXP / totalXP) * 100).."%") or "0%"
			end
		end,
		events = "UPDATE_EXHAUSTION",
		order = 64,
		name = L["Percent Rested XP"],
		desc = L["Rested XP as a percentage of the current level"],
	},
	["currep"] = {
		func = function(u)
			local repname, repreaction, repmin, repmax, repvalue = GetWatchedFactionInfo()
			if repname then
				repvalue = repvalue - repmin
				repmax = repmax - repmin
				return repvalue..'/'..repmax
			end
			return ''
		end,
		events = "PLAYER_XP_UPDATE",
		order = 66,
		name = L["Current Reputation"],
		desc = L["Reputation with the currently watched faction"],
	},
	["percentrep"] = {
		func = function(u)
			local repname, repreaction, repmin, repmax, repvalue = GetWatchedFactionInfo()
			if repname then
				repvalue = repvalue - repmin
				repmax = repmax - repmin
				return (repmax > 0) and (floor((repvalue / repmax) * 100)..'%') or '0%'
			end
			return ''
		end,
		events = "PLAYER_XP_UPDATE",
		order = 67,
		name = L["Percent Reputation"],
		desc = L["Reputation with the currently watched faction as a percentage"],
	},
}

function aUF:FormatString(str)
	if not (type(str) == "string") then str = nil return end

	if aUF.formats[str] and aUF.formats[str].func then
		return aUF.formats[str].func, aUF.formats[str].events
	end

	local formatArgs, formatEvents = new(), new()
	for word in string.gmatch(str, "%a+") do
		if aUF.textFunctions[word] and aUF.textFunctions[word].func then
			table.insert(formatArgs, aUF.textFunctions[word].func)
			local events = aUF.textFunctions[word].events
			if events then
				if type(events) == "table" then
					for k,v in pairs(events) do
						table.insert(formatEvents, v)
					end
				elseif type(events) == "string" then
					table.insert(formatEvents, events)
				end
			end
		end
	end

	if not aUF.formats[str] then
		aUF.formats[str] = new()
	end

	aUF.formats[str].func = function(unit)
		local work = new()
		local n = 1
		local str
		for i,func in ipairs(formatArgs) do
			local result = func(unit)
			if result then
				work[n] = func(unit)
				n = n + 1
			end
		end
		local str = table.concat(work, " ")
		work = del(work)
		return str
	end
	aUF.formats[str].events = formatEvents

	return aUF.formats[str].func, aUF.formats[str].events
end
