local L = LibStub("AceLocale-3.0"):GetLocale("ag_UnitFrames", true)
local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")
local AceDB = LibStub("AceDB-3.0")
local media = LibStub("LibSharedMedia-3.0")

local font = L["Interface\\AddOns\\ag_UnitFrames\\fonts\\barframes.ttf"]
local newFrame, delFrame = aUF.newFrame, aUF.delFrame

local ABF = aUF:NewModule("ABFlayout")

aUF.layouts.ABF = ABF

local defaults = {
	profile = {
		["**"] = {
			HealthBarLeftText = "name afkdnd",
			ManaBarLeftText = "mobtype level class race",	
			HealthBarRightText = "percenthp",
			ManaBarRightText = "curmp",
			height = {
				HealthBar = 1.2,
				ManaBar = 1.0,
				CastBar = 0.8,
				XPBar = 0.4,
			},
			order = {
				HealthBar = 1,
				ManaBar = 2,
				XPBar = 3,
				CastBar = 4,
			},
		},
		target = {
			HealthBarRightText = "percenthp curhp",
		},
		player = {
			HealthBarRightText = "percenthp",
		},
		pet = {
			HealthBarRightText = "percenthp",
		},
		party = {
			HealthBarRightText = "percenthp",
		}		
	}	
}

local order = {
	HealthBar = 1,
	ManaBar = 2,
	XPBar = 3,
	CastBar = 4,
}

function ABF:OnRegister()
	ABF.db = aUF.db:RegisterNamespace("ABFlayout", defaults)
end

function ABF:LoadScale(object)
	object.frame:SetScale(object.database.Scale)
end

function ABF:BorderBackground(object)
	local colortable = aUF.db.profile.PartyFrameColors
	local bordercolor = aUF.db.profile.FrameBorderColors
	local borderstyle = object.database.BorderStyle or aUF.db.profile.BorderStyle
	
	object.frame:SetBackdrop({
				bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
				edgeFile = aUF.Borders[borderstyle].texture, edgeSize = aUF.Borders[borderstyle].size,
				insets = {left = aUF.Borders[borderstyle].insets, right = aUF.Borders[borderstyle].insets, top = aUF.Borders[borderstyle].insets, bottom = aUF.Borders[borderstyle].insets},
		})

	object.frame:SetBackdropColor(colortable.r,colortable.g,colortable.b,colortable.a)
	object.frame:SetBackdropBorderColor(bordercolor.r,bordercolor.g,bordercolor.b,bordercolor.a)
end

function ABF:ApplyLayout(object)
	local db = ABF.db.profile
	local mFont
	
	if object.database.fontOverrideType then
		mFont = media:Fetch("font", object.database.fontOverrideType)
	end
	
	--special frames
	if object.Portrait then
		object.Portrait:SetWidth(12+0.15*object.frame:GetWidth())
	end
	
	if object.Totems then
		object.Totems:SetPoint("BOTTOMLEFT", object.frame, "TOPLEFT", 4, -4)
	end
	
	local fontSize
	if object.HitIndicator then
		if mFont then
			fontSize = (object.database.fontOverride and (object.database.fontOverrideSize or 0) + 16) or 16
		end	
		object.HitIndicator:ClearAllPoints()
		object.HitIndicator:SetFont(mFont or font, fontSize)
		object.HitIndicator:SetPoint("CENTER",object.frame)
	end
	
	if object.Combo1 then
		object.Combo1:ClearAllPoints()
		object.Combo1:SetPoint("BOTTOMRIGHT",object.frame,"BOTTOMRIGHT")
	end
	
	-- icons
	local icon = object.icons.pvpIcon
	if icon then
		icon:ClearAllPoints()
		icon:SetPoint("TOPRIGHT",object.frame,"TOPRIGHT",11,2)
		icon:SetWidth(23)
		icon:SetHeight(23)
	end
	icon = object.icons.statusIcon
	if icon then
		icon:ClearAllPoints()
		icon:SetPoint("BOTTOMLEFT",object.frame,"BOTTOMLEFT",-4,-3)
		icon:SetWidth(18)
		icon:SetHeight(18)
	end
	icon = object.icons.leaderIcon
	if icon then
		icon:ClearAllPoints()
		icon:SetPoint("TOPLEFT",object.frame,"TOPLEFT",-2,2)
		icon:SetWidth(12)
		icon:SetHeight(12)
	end
	icon = object.icons.looterIcon
	if icon then
		icon:ClearAllPoints()
		icon:SetPoint("TOPLEFT",object.frame,"TOPLEFT",7,2)
		icon:SetWidth(11)
		icon:SetHeight(11)
	end
	icon = object.icons.raidTargetIcon
	if icon then
		icon:ClearAllPoints()
		icon:SetPoint("CENTER",object.frame,"TOP",0,-3)
		icon:SetWidth(18)
		icon:SetHeight(18)
	end
	icon = object.icons.HappinessIcon
	if icon then
		icon:ClearAllPoints()
		icon:SetPoint("BOTTOMRIGHT",object.frame,"BOTTOMRIGHT",4,-3)
		icon:SetWidth(15)
		icon:SetHeight(15)
	end
	
	-- bars
	local numBars = 0
	for k,v in pairs(object.bars) do
		numBars = numBars + 1
	end
	local fHeight = object.frame:GetHeight() - 10 - numBars + 1
	
	local totalWeight = 0
	local str,str2
	local anchor
	
	local sortTbl = {}
	for k,v in pairs(object.bars) do
		table.insert(sortTbl, k)
		totalWeight = totalWeight + (db[object.type].height[k] or 1)
	end
	
	local textSort = function(alpha, bravo)
		return (db[object.type].order[alpha] or 999) > (db[object.type].order[bravo] or 999)
	end	
	table.sort(sortTbl, textSort)
	
	local LEFT = "LEFT"
	local RIGHT = "RIGHT"
	local offset = 1
	if string.find(object.type, "target") then
		LEFT = "RIGHT"
		RIGHT = "LEFT"
		offset = -1
	end
	
	local v
	local anchoredPortrait
	local tag
--	aUF:Print(object.type)
	for i,k in pairs(sortTbl) do
		local v = object.bars[k]
		v:ClearAllPoints()
		
		if anchor then
			if object.Portrait and not (anchoredPortrait) and (order[k] or 999) < 3 then
				v:SetPoint("BOTTOM"..LEFT,object.Portrait,"BOTTOM"..RIGHT, 1*offset, 0)
				v:SetPoint("BOTTOM"..RIGHT,anchor,"TOP"..RIGHT, 0, 1)
				anchoredPortrait = true
				object.Portrait:ClearAllPoints()
				object.Portrait:SetPoint("BOTTOM"..LEFT,anchor,"TOP"..LEFT, 0, 1)
				object.Portrait:SetPoint("TOP"..LEFT,object.frame,"TOP"..LEFT, 5*offset, -5)
			else
				v:SetPoint("BOTTOM"..LEFT,anchor,"TOP"..LEFT, 0, 1)
				v:SetPoint("BOTTOM"..RIGHT,anchor,"TOP"..RIGHT, 0, 1)
			end
		else
			if object.Portrait and not (anchoredPortrait) and (order[k] or 999) < 3 then
				v:SetPoint("BOTTOM"..LEFT,object.Portrait,"BOTTOM"..RIGHT, 1*offset, 0)
				v:SetPoint("BOTTOM"..RIGHT,object.frame,"BOTTOM"..RIGHT, -5*offset, 5)
				anchoredPortrait = true
				object.Portrait:ClearAllPoints()
				object.Portrait:SetPoint("BOTTOM"..LEFT,object.frame,"BOTTOM"..LEFT, 5*offset, 5)
				object.Portrait:SetPoint("TOP"..LEFT,object.frame,"TOP"..LEFT, 5*offset, -5)
			else
				v:SetPoint("BOTTOM"..LEFT,object.frame,"BOTTOM"..LEFT, 5*offset, 5)
				v:SetPoint("BOTTOM"..RIGHT,object.frame,"BOTTOM"..RIGHT, -5*offset, 5)
			end
		end
		
		v:SetHeight(fHeight*((db[object.type].height[k] or 1)/totalWeight))
		v:Show()
		
		if i == #sortTbl then
			fontSize = 11
		else
			fontSize = 10
		end
		if mFont then
			fontSize = (object.database.fontOverride and (object.database.fontOverrideSize or 0) + fontSize) or fontSize
		end
		
		-- Strings that uses tags must be put into object.strings !
		if v.text2 then
			str2 = v.text2
		else		
			if not object.strings[k.."RightText"] then
				object.strings[k.."RightText"] = newFrame("FontString", object.top, "OVERLAY")
				object.strings[k.."RightText"].layout = self
			end
--			aUF:Print(k.."RightText")
			str2 = object.strings[k.."RightText"]
			str2:Show()
			tag = (self.db.profile and self.db.profile[object.type] and self.db.profile[object.type][k.."RightText"])
			object:RegisterString(str2,tag,k.."RightText")
		end
		str2:SetShadowColor(0, 0, 0)
		str2:SetShadowOffset(0.8, -0.8)
		str2:SetPoint("RIGHT",v,"RIGHT",-3,0.5)
		str2:SetFont(mFont or font, fontSize)
		str2:SetJustifyH("RIGHT")
		if k == "HealthBar" then
			object.statusText = str2
		end
		
		if v.text1 then
			str = v.text1
		else
			if not object.strings[k.."LeftText"] then
				object.strings[k.."LeftText"] = newFrame("FontString", object.top, "OVERLAY")
				object.strings[k.."RightText"].layout = self
			end
			str = object.strings[k.."LeftText"]
			str:Show()
			tag = (self.db.profile and self.db.profile[object.type] and self.db.profile[object.type][k.."LeftText"])
			object:RegisterString(str,tag,k.."LeftText")
		end
		str:SetShadowColor(0, 0, 0)
		str:SetShadowOffset(0.8, -0.8)
		str:SetPoint("LEFT",v,"LEFT",3,0.5)
		str:SetPoint("RIGHT",str2,"LEFT",3,0.5)
		str:SetFont(mFont or font, fontSize)
		str:SetJustifyH("LEFT")
		
		anchor = v
	end
	
	if object.Portrait and not (anchoredPortrait) then
		object.Portrait:SetPoint("BOTTOMLEFT",object.frame,"BOTTOMLEFT", 5, 5)
	end
end

function ABF:UnapplyLayout(object)
	for k,v in pairs(object.strings) do
		if v.layout == self then
			object:UnregisterString(v, k)		
			object.strings[k] = delFrame(v)
		end
	end
end

ABF.auras = {
	Left = {
		Point = "RIGHT",
		RelativePoint = "LEFT",
		x = 0,
		y = 0,
		GrowDirection = "LEFT",
		Scale = 0.80,
		Alpha = 1,
	},
	Right = {
		Point = "LEFT",
		RelativePoint = "RIGHT",
		x = 0,
		y = 0,
		GrowDirection = "RIGHT",
		Scale = 0.80,
		Alpha = 1,
	},
	Above = {
		Point = "BOTTOMLEFT",
		RelativePoint = "TOPLEFT",
		x = 3,
		y = -0.5,
		GrowDirection = "RIGHT",
		Scale = "auto",
		AdjustAuto = 7,
		Alpha = 1,
	},
	Below = {
		Point = "TOPLEFT",
		RelativePoint = "BOTTOMLEFT",
		x = 3,
		y = 0.5,
		GrowDirection = "RIGHT",
		Scale = "auto",
		AdjustAuto = 7,
		Alpha = 1,
	},
	Inside = {
		Point = "TOPRIGHT",
		RelativePoint = "TOPRIGHT",
		x = -6,
		y = -6,
		GrowDirection = "LEFT",
		Scale = 0.7,
		Alpha = 0.75,
	}
}