local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")
local aUF_auras = aUF:NewModule("Auras")
local newFrame, delFrame = aUF.newFrame, aUF.delFrame

-- QueueSet: A FIFO Queue which guarantees all elements to be unique
local QueueSet = {};

function QueueSet:new()
	return {
		offer = self.offer,
		poll = self.poll,
		peek = self.peek,
		elements = {},
		map = {},
	};
end

function QueueSet:offer(e)
	if (self.map[e]) then return end
	self.map[e] = true
	self.elements[#self.elements + 1] = e
end

function QueueSet:poll() 
	local e = table.remove(self.elements, 1)
	if (e ~= nil) then
		if (not self.map[e]) then error("invalid state in QueueSet") end
		self.map[e] = nil
	end
	return e
end

function QueueSet:peek()
	return self.elements[1]
end

local auraUpdatePool = QueueSet:new()

local CanDispel = {
	PRIEST = {
		Magic = true,
		Disease = true,
	},
	SHAMAN = {
		Poison = true,
		Disease = true,
	},
	PALADIN = {
		Magic = true,
		Poison = true,
		Disease = true,
	},
	MAGE = {
		Curse = true,
	},
	DRUID = {
		Curse = true,
		Poison = true,
	}
}
	
function aUF_auras:OnRegister()
	self:RegisterDefaults({
		units = {
			["**"] = {
				AuraStyle = "TwoLines",
				AuraPos = "Below",
				AuraFilter = 0,
				AuraDebuffC = false,
				AuraRows = 2,
				AuraColumns = 10,
				AuraPreferBuff = true,
				cooldown = true,
			},
			["raid"] = {
				AuraPos = "Hidden",
				AuraRows = 1,
				AuraFilter = 1,
				AuraDebuffC = true,				
			},
			["player"] = {
				AuraFilter = 1,
				AuraDebuffC = true,				
			},
			["pet"] = {
				AuraFilter = 1,
				AuraDebuffC = true,				
			},			
			["party"] = {
				AuraFilter = 1,
				AuraDebuffC = true,				
			},	
			["partypet"] = {
				AuraRows = 1,
				AuraColumns = 8,
				AuraFilter = 1,
				AuraDebuffC = true,				
			},
			["targettarget"] = {
				AuraRows = 1,
				AuraColumns = 8,
			},
			["partytarget"] = {
				AuraPos = "Hidden",
			},			
		}
	})
	
	self.OnUpdateEvent = self:ScheduleRepeatingTimer("UpdatePool", 0.05)
	if (not self.cache) then 
		self.cache = {}
	end
	self.cache.buffs = {}
	self.cache.debuffs = {}

	for i = 1,50 do
		self.cache.buffs[i] = {}
		self.cache.debuffs[i] = {}
	end	
end

function aUF_auras:OnEnable()
	for _,object in pairs(aUF.units) do
		if object.type then
			self:OnParseLayout(object)
			if object.unit then
				object:SetAuraCount()
				object:UpdateAuras()
				self:OnRegisterEvents(object)
			end
		end
	end
end

function aUF_auras:OnDisable()
	for _,object in pairs(aUF.units) do
		if object.type then
			self:OnRegisterEvents(object)
			local i = 1
			while (true) do
				local f = object["Aura"..i]
				if (not f) then
					break
				end
				f:Hide()
				i = i+1
			end
		end
	end
end

function aUF_auras:OnTypeReset(object)
	object:SetAuraCount()
end

function aUF_auras:OnRegisterEvents(object)
	local position = object.database.AuraPos
	local auraDebuffC = object.database.AuraDebuffC
	local hide = position == "Hidden"
	if ((not (hide)) or auraDebuffC) and self:IsEnabled() then
		object:ObjectRegisterEvent("UNIT_AURA", "UpdateAuras")
	else
		object:ObjectUnregisterEvent("UNIT_AURA", "UpdateAuras")
	end
end

function aUF_auras:OnDisableObject(object)
end

function aUF_auras:OnMetroUpdate(object)
	object:UpdateAuras()
end

function aUF_auras:OnUpdateAll(object)
	object:UpdateAuras()
end

function aUF_auras:UpdatePool()
	for i = 1,5 do
		local object = auraUpdatePool:poll()
		if object then
			object:UpdateAuras(nil,nil,true)
		else
			break
		end
	end
end

function aUF_auras:OnSetWidth(object)
	object:AuraPosition()
end

function aUF_auras:OnParseLayout(object)
	if not object.DebuffHighlight then
		local highlight = newFrame("Texture",object.top, "OVERLAY")
		highlight:SetAlpha(0.5)
		highlight:SetTexture("Interface\\AddOns\\ag_UnitFrames\\Images\\debuffHighlight")
		highlight:SetBlendMode("ADD")	
		highlight:SetPoint("TOPLEFT",object.frame, "TOPLEFT", 5, -5)
		highlight:SetPoint("BOTTOMRIGHT",object.frame, "BOTTOMRIGHT", -5, 5)
		highlight:Hide()
		object.DebuffHighlight = highlight
	end
end

-----------
-- AURAS --
-----------

local function hideTT()
	GameTooltip:Hide()
end

local function onAuraEnter()
	if (not this:IsVisible()) then return end
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
	if ( this.buffFilter == "HELPFUL") then
		GameTooltip:SetUnitBuff(this.unit.unit, this.id, this.unit.database.AuraFilter)
	elseif ( this.buffFilter == "HARMFUL") then
		GameTooltip:SetUnitDebuff(this.unit.unit, this.id, 0)
	end
end

function aUF.classes.aUFunit.prototype:SetAuraCount()
	local db = self.database
	self.AuraRows, self.AuraColumns = db.AuraRows, db.AuraColumns

	local count = self.AuraRows * self.AuraColumns

	-- setup auras
	for i = 1,math.max(count) do
		local aura = "Aura" .. i
		if (not self[aura]) then
			local name = self.name .. "_" .. aura
			self[aura] = newFrame("Button", self.middle)
			self[aura]:Hide()			
			self[aura]:SetWidth(15)
			self[aura]:SetHeight(15)
			
			self[aura].Icon = newFrame("Texture", self[aura], "BACKGROUND")
			self[aura].Icon:SetAllPoints(self[aura])
			
			self[aura].Overlay = newFrame("Texture", self[aura], "ARTWORK")
			self[aura].Overlay:SetPoint("CENTER", self[aura])
			self[aura].Overlay:SetTexture("Interface\\AddOns\\ag_UnitFrames\\Images\\UI-Debuff-Overlays")
			self[aura].Overlay:SetWidth(16)
			self[aura].Overlay:SetHeight(16)
			
			self[aura].Count = newFrame("FontString", self[aura], "OVERLAY")
			self[aura].Count:SetFont("Interface\\AddOns\\ag_UnitFrames\\fonts\\barframes.ttf", 10, "OUTLINE")
			self[aura].Count:SetShadowColor(0, 0, 0, 1)
			self[aura].Count:SetShadowOffset(0.8, -0.8)
			self[aura].Count:SetPoint("BOTTOMRIGHT", self[aura], "BOTTOMRIGHT", 1, 0)
			self[aura].Count:SetWidth(18)
			self[aura].Count:SetHeight(10)
			self[aura].Count:SetJustifyH("RIGHT")

			self[aura].cooldown = newFrame("cooldown", self[aura])
			self[aura].cooldown:Hide()			
			self[aura].cooldown:SetAllPoints(self[aura])
			self[aura].cooldown:SetReverse(true)
			self[aura].cooldown:SetFrameLevel(self[aura]:GetFrameLevel())
			self[aura].cooldown:SetAlpha(0.8)
			
			self[aura]:SetScript("OnEnter", onAuraEnter)
			self[aura]:SetScript("OnLeave", hideTT)
			self[aura].unit = self
		end
	end
	local i = count+1
	while (true) do
		local f = self["Aura"..i]
		if (not f) then
			break
		end
		f:Hide()
		i = i+1
	end
end

function aUF.classes.aUFunit.prototype:AuraPosition()
	if not self.frame:GetWidth() or 0 >= self.frame:GetWidth() then
		return
	end
	local position = self.database.AuraPos
	local count = self.AuraRows * self.AuraColumns

	self.Aura1:ClearAllPoints()

	local Point, RelativePoint,x,y,GrowDirection,Scale,Alpha,AdjustAuto
	local auraTable
	if aUF.layouts[self.database.FrameStyle] then
		auraTable = aUF.layouts[self.database.FrameStyle].auras or aUF.layouts.ABF.auras
	else
		return
	end

	if auraTable[position] then
		Point = auraTable[position].Point
		RelativePoint = auraTable[position].RelativePoint
		x = auraTable[position].x
		y = auraTable[position].y
		GrowDirection = auraTable[position].GrowDirection
		Scale = auraTable[position].Scale
		Alpha = auraTable[position].Alpha
		AdjustAuto = auraTable[position].AdjustAuto
	else
		return
	end

	if Scale == "auto" then
		Scale = (self.frame:GetWidth()-AdjustAuto)/(16*self.AuraColumns)
		self.Aura1:SetPoint(Point,self.frame,RelativePoint, x/Scale, y/Scale)
	else
		self.Aura1:SetPoint(Point,self.frame,RelativePoint, x*(1/Scale), y*(1/Scale))
	end

	for i=1,count do
		self["Aura"..i]:SetScale(Scale)
		self["Aura"..i]:SetAlpha(Alpha)
	end

	if ( GrowDirection == "LEFT" ) then
		for i=2,count do
			self["Aura"..i]:ClearAllPoints()
			if (math.fmod(i,self.AuraColumns) == 1) then
				self["Aura"..i]:SetPoint("TOP",self["Aura"..i-self.AuraColumns],"BOTTOM",0,-1)
			else
				self["Aura"..i]:SetPoint("RIGHT",self["Aura"..i-1],"LEFT",1,0)
			end
		end
	else
		for i=2,count do
			self["Aura"..i]:ClearAllPoints()
			if (i % self.AuraColumns == 1) then
				if position == "Above" then
					self["Aura"..i]:SetPoint("BOTTOM",self["Aura"..i-self.AuraColumns],"TOP",0,1)
				else
					self["Aura"..i]:SetPoint("TOP",self["Aura"..i-self.AuraColumns],"BOTTOM",0,-1)
				end
			else
				self["Aura"..i]:SetPoint("LEFT",self["Aura"..i-1],"RIGHT",1,0)
			end
		end
	end
end

function aUF.classes.aUFunit.prototype:UpdateAuras(_,_,exec)
	if not exec then
		auraUpdatePool:offer(self)
		return
	end
	local filter, position, AuraDebuffC, cooldown = self.database.AuraFilter, self.database.position, self.database.AuraDebuffC, self.database.cooldown
	local position = self.database.AuraPos
	local buttons, dFound
	local rows, cols, auracount = self.AuraRows, self.AuraColumns, self.AuraRows * self.AuraColumns

	local hide = position == "Hidden"
	local hidef = self["Aura1"].Hide

	-- hide all auras
	if (not self.hidden) then
		for i=1,auracount do
			hidef(self["Aura"..i])
		end

		self.hidden = true
	end

	if (not self.unit) then
		return
	end
	
	-- scan for buffs and debuffs
	local bcount, dbcount, highlighttype = self:ScanAuras()
	if (highlighttype and DebuffTypeColor[highlighttype] and AuraDebuffC == true) then
		local h = DebuffTypeColor[highlighttype]
		self.DebuffHighlight:Show()
		self.DebuffHighlight:SetVertexColor(h.r, h.g, h.b)
	else
		self.DebuffHighlight:Hide()
	end
	
	if hide or bcount + dbcount == 0 then
		self.hidden = true
		return
	end
	self.hidden = false

	if (auracount < bcount + dbcount) then
		if (self.database.AuraPreferBuff) then
			dbcount = auracount - bcount
			if (dbcount < 0) then
				bcount = auracount
				dbcount = 0
			end
		else
			bcount = auracount - dbcount
			if (bcount < 0) then
				dbcount = auracount
				bcount = 0
			end
		end
	end

	local buff, count, class, index, c, id

	for i=1,bcount do
		c = aUF_auras.cache.buffs[i];
		buff, count, id, timeLeft, duration = c.texture, c.count, c.id, cooldown and c.timeLeft, c.duration
		self:SetAura(i, false, id, buff, count, nil, timeLeft, duration)
	end

	rows = math.ceil((bcount+dbcount) / cols)

	if (rows == 1 and (position == "Left" or position == "Right" or position == "Inside")) then
		if (cols > bcount+dbcount) then
			cols = bcount+dbcount+0
		end
	end

	for i=1,dbcount do
		c = aUF_auras.cache.debuffs[i];
		buff, count, class, id, timeLeft, duration = c.texture, c.count, c.type, c.id, cooldown and c.timeLeft, c.duration
		index = rows * cols - i + 1
		self:SetAura(index, true, id, buff, count, class, timeLeft, duration)
	end
end

function aUF.classes.aUFunit.prototype:SetAura(index, isDebuff, id, buff, count, class, timeLeft, duration)
	local buffFrame = self["Aura" .. index]
	buffFrame.Icon:SetTexture(buff)
	buffFrame:Show()
	buffFrame.id = id
	
	if duration and duration > 0 and timeLeft and timeLeft > 0 then
		local startCooldownTime = GetTime() + timeLeft - duration
		buffFrame.cooldown:SetCooldown(startCooldownTime, duration)
		buffFrame.cooldown:Show()
	else
		buffFrame.cooldown:Hide()
	end
	
	if count and count > 1 then
		buffFrame.Count:SetText(count)
	else
		buffFrame.Count:SetText("")
	end

	if (isDebuff) then
		buffFrame.buffFilter = "HARMFUL"
		local borderColor
		if class then
			borderColor = DebuffTypeColor[class]
		else
			borderColor = DebuffTypeColor["none"]
		end
		buffFrame.Overlay:SetVertexColor(borderColor.r, borderColor.g, borderColor.b)
		buffFrame.Overlay:SetAlpha(1)
	else
		buffFrame.Overlay:SetVertexColor(0.6, 0.9, 0.6)
		buffFrame.Overlay:SetAlpha(0.4)
		buffFrame.buffFilter = "HELPFUL"
	end
end

function aUF.classes.aUFunit.prototype:ShowAuraPositions()
	local count = self.AuraRows*self.AuraColumns
	if (self.database.AuraPreferBuff) then
		for i = 1,count do
			local b = self["Aura"..i]
			if (i <= 16) then
				b.Icon:SetTexture("Interface\\Icons\\Spell_ChargePositive")
			else
				b.Icon:SetTexture("Interface\\Icons\\Spell_ChargeNegative")
			end
		end
	else
		for i = 1,count do
			local b = self["Aura"..i]
			if (i > count-16) then
				b.Icon:SetTexture("Interface\\Icons\\Spell_ChargeNegative")
			else
				b.Icon:SetTexture("Interface\\Icons\\Spell_ChargePositive")
			end
		end
	end
	for i = 1,count do
		local b = self["Aura"..i]
		b:Show()
		b.Overlay:Show()
	end

	self.hidden = false
	self:ScheduleEvent(self.UpdateAuras, 10, self)
end

function aUF.classes.aUFunit.prototype:ScanAuras()
	local buffid, id, unit = 1, 1, self.unit
	local c, name, rank, texture, debufftype, applications, logdb
	local filter = self.database.AuraFilter
	local dbcount, bcount
	
	-- record buffs into a temporary area
	-- no special cases needed, just store the data
	name, rank, texture, applications, duration, timeLeft = UnitBuff(unit, buffid, filter)
	while (name) do
		-- record buffs
		c = aUF_auras.cache.buffs[buffid]
		c.id, c.name, c.texture, c.count, c.duration, c.timeLeft = buffid, name, texture, applications, duration, timeLeft

		buffid = buffid + 1
		name, rank, texture, applications, duration, timeLeft = UnitBuff(unit, buffid, filter)
	end

	bcount = buffid - 1

	-- record debuffs
	-- special case: priest weakened soul debuff
	local _, playerclass = UnitClass("player")
	id, buffid = 1, 1
	name, rank, texture, applications, debufftype, duration, timeLeft = UnitDebuff(unit, buffid)

	while (name) do
		-- record if filter is okay
		if (filter == 0) then
			logdb = true
		elseif (CanDispel[playerclass] and CanDispel[playerclass][debufftype]) then
			logdb = true
		elseif (playerclass == "PRIEST" and texture == "Interface\\Icons\\Spell_Holy_AshesToAshes") then
			logdb = true
		else
			logdb = false
		end

		if (logdb) then
			c = aUF_auras.cache.debuffs[id]
			c.id, c.name, c.texture, c.count, c.type, c.duration, c.timeLeft = buffid, name, texture, applications, debufftype, duration, timeLeft
			id = id + 1
		end

		buffid = buffid + 1
		name, rank, texture, applications, debufftype, duration, timeLeft = UnitDebuff(unit, buffid)
	end

	dbcount = id - 1

	if (dbcount > 0) then
		return bcount, dbcount, aUF_auras.cache.debuffs[1].type
	else
		return bcount, dbcount
	end
end