local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")
local aUF_statusicon = aUF:NewModule("Statusicon")
local newFrame, delFrame = aUF.newFrame, aUF.delFrame

local combatStatus = {}

function aUF_statusicon:OnInitialize()
	self:RegisterDefaults({
		units = {
			["**"] = {
				statusIcon = true,
			},		
		}
	})
end

local pool = {}
function aUF_statusicon:OnRegisterEvents(object)
	if self:IsEnabled() and object.database.statusIcon then
		if object.type == "player" then
			object:ObjectRegisterEvent("PLAYER_REGEN_ENABLED", "UpdateStatusIcon")
			object:ObjectRegisterEvent("PLAYER_REGEN_DISABLED", "UpdateStatusIcon")
			object:ObjectRegisterEvent("PLAYER_UPDATE_RESTING", "UpdateStatusIcon")
			object:ObjectRegisterEvent("PLAYER_ENTERING_WORLD", "UpdateStatusIcon")
		else
			if not self.inCombatSchedule then
				self.inCombatSchedule = self:ScheduleRepeatingTimer("UpdatePool", 0.5)
			end
			pool[object] = true
		end
	else
		if object.type == "player" then
			object:ObjectUnregisterEvent("PLAYER_REGEN_ENABLED", "UpdateStatusIcon")
			object:ObjectUnregisterEvent("PLAYER_REGEN_DISABLED", "UpdateStatusIcon")
			object:ObjectUnregisterEvent("PLAYER_UPDATE_RESTING", "UpdateStatusIcon")
			object:ObjectUnregisterEvent("PLAYER_ENTERING_WORLD", "UpdateStatusIcon")
		else
			pool[object] = nil
		end
	end
end

function aUF_statusicon:OnUpdateAll(object)
	object:UpdateStatusIcon()
end

function aUF_statusicon:OnMetroUpdate(object)
	object:UpdateStatusIcon()
end

function aUF_statusicon:UpdatePool()
	if next(pool) == nil and self.inCombatSchedule then
		self:CancelTimer(self.inCombatSchedule)
		self.inCombatSchedule = nil
	end
	for object in pairs(pool) do
		object:UpdateStatusIcon()
	end
end

function aUF_statusicon:OnEnable()
	for _,object in pairs(aUF.units) do
		if object.type then
			object:ApplyLayout()
			if object.unit then
				object:UpdateStatusIcon()
				self:OnRegisterEvents(object)
			end
		end
	end
end

function aUF_statusicon:OnDisable()
	for _,object in pairs(aUF.units) do
		if object.type then
			if object.icons.statusIcon then
				object.icons.statusIcon = delFrame(object.icons.statusIcon)		
				self:OnRegisterEvents(object)
				object:ApplyLayout()
			end
		end
	end
end

function aUF_statusicon:OnParseLayout(object)
	if not object.icons.statusIcon then
		object.icons.statusIcon = newFrame("Texture", object.top, "OVERLAY")
		object.icons.statusIcon:SetTexture("Interface\\CharacterFrame\\UI-StateIcon")
	end
end

function aUF.classes.aUFunit.prototype:UpdateStatusIcon()
	if self.database.statusIcon then
		if self.unit and UnitAffectingCombat(self.unit) then
			self.icons.statusIcon:Show()
			self.icons.statusIcon:SetTexCoord(0.5,1,0,0.49)
			return
		elseif self.unit == "player" and IsResting() then
			self.icons.statusIcon:Show()
			self.icons.statusIcon:SetTexCoord(0,0.5,0,0.421875)
			return
		end
	end
	self.icons.statusIcon:Hide()
end

