local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")
local aUF_combattext = aUF:NewModule("Combattext")
local newFrame, delFrame = aUF.newFrame, aUF.delFrame

local font = "Fonts\\FRIZQT__.TTF"
local maxalpha = 0.6
local feedback = {}

setmetatable(feedback, { __newindex = function(self,k,v)
	rawset(self, k, v)
	if not aUF_combattext.schedule then
		aUF_combattext.schedule = aUF.ScheduleRepeatingTimer(aUF_combattext, "UpdatePool", 0.05)
	end
end })	

function aUF_combattext:OnRegister()
	self:RegisterDefaults({
		units = {
			["**"] = {
				ShowCombat = true,
			},		
		}
	})	
end

function aUF_combattext:OnRegisterEvents(object)
	if object.database.ShowCombat and self:IsEnabled() then
		object:ObjectRegisterEvent("UNIT_COMBAT","UnitCombat")
		object:ObjectRegisterEvent("UNIT_SPELLMISS","UnitSpellmiss")
	else
		object:ObjectUnregisterEvent("UNIT_COMBAT","UnitCombat")
		object:ObjectUnregisterEvent("UNIT_SPELLMISS","UnitSpellmiss")
	end
end

function aUF_combattext:OnDisableObject(object)
end

function aUF_combattext:OnUpdateAll(object)
end

function aUF_combattext:OnEnable()
	for _,object in pairs(aUF.units) do
		if object.type then
			object:ApplyLayout()
			if object.unit then
				self:OnRegisterEvents(object)			
			end
		end
	end
end

function aUF_combattext:OnDisable()
	for _,object in pairs(aUF.units) do
		if object.HitIndicator then
			object.HitIndicator = delFrame(object.HitIndicator)
			self:OnRegisterEvents(object)
			feedback[object] = nil
		end
	end
end

function aUF_combattext:UpdatePool()
	if next(feedback) == nil and self.schedule then
		self:CancelTimer(self.schedule)
		self.schedule = nil
		return
	end
	for object in pairs(feedback) do
        local elapsedTime = GetTime() - object.feedbackStartTime
        if ( elapsedTime < COMBATFEEDBACK_FADEINTIME ) then
            local alpha = maxalpha*(elapsedTime / COMBATFEEDBACK_FADEINTIME)
            object.HitIndicator:SetAlpha(alpha)
        elseif ( elapsedTime < (COMBATFEEDBACK_FADEINTIME + COMBATFEEDBACK_HOLDTIME) ) then
            object.HitIndicator:SetAlpha(maxalpha)
        elseif ( elapsedTime < (COMBATFEEDBACK_FADEINTIME + COMBATFEEDBACK_HOLDTIME + COMBATFEEDBACK_FADEOUTTIME) ) then
            local alpha = maxalpha - maxalpha*((elapsedTime - COMBATFEEDBACK_HOLDTIME - COMBATFEEDBACK_FADEINTIME) / COMBATFEEDBACK_FADEOUTTIME)
            object.HitIndicator:SetAlpha(alpha)
        else
            object.HitIndicator:Hide()
            feedback[object] = nil
        end
	end
end

function aUF_combattext:OnParseLayout(object)
	if object.database.ShowCombat then
		if not object.HitIndicator then
			object.HitIndicator = newFrame("FontString", object.top, "OVERLAY")
			object.HitIndicator:SetShadowColor(0, 0, 0)
			object.HitIndicator:SetShadowOffset(0.8, -0.8)
		end
	elseif object.HitIndicator then
		object.HitIndicator = delFrame(object.HitIndicator)		
	end
end

-------------------
-- COMBAT EVENTS --
-------------------

function aUF.classes.aUFunit.prototype:UnitCombat(event, unit)
	if unit ~= self.unit then return end
	if not ( self.database.ShowCombat ) then return end    
	self:CombatFeedback_OnCombatEvent(arg1, arg2, arg3, arg4, arg5)
end

function aUF.classes.aUFunit.prototype:UnitSpellmiss(event, unit)
	if unit ~= self.unit then return end
	if not ( self.database.ShowCombat ) then return end
	self:CombatFeedback_OnSpellMissEvent(arg1, arg2)
end

function aUF.classes.aUFunit.prototype:CombatFeedback_OnCombatEvent(unit, event, flags, amount, type)
	if not self.HitIndicator then return end
	local fontHeight = 13
	local text = ""
	local r,g,b = 1,1,1
	if( event == "IMMUNE" ) then
		fontHeight = fontHeight * 0.75
		text = CombatFeedbackText[event]
	elseif ( event == "WOUND" ) then
		if ( amount ~= 0 ) then
			if ( flags == "CRITICAL" or flags == "CRUSHING" ) then
				fontHeight = fontHeight * 1.5
			elseif ( flags == "GLANCING" ) then
				fontHeight = fontHeight * 0.75
			end
			if ( type > 0 ) then
				r = 1.0
				g = 1.0
				b = 0.0
			end
			if UnitInParty(self.unit) or UnitInRaid(self.unit) then
				r = 1.0
				g = 0.0
				b = 0.0
			end
			text = "-"..amount
		elseif ( flags == "ABSORB" ) then
			fontHeight = fontHeight * 0.75
			text = CombatFeedbackText["ABSORB"]
		elseif ( flags == "BLOCK" ) then
			fontHeight = fontHeight * 0.75
			text = CombatFeedbackText["BLOCK"]
		elseif ( flags == "RESIST" ) then
			fontHeight = fontHeight * 0.75
			text = CombatFeedbackText["RESIST"]
		else
			text = CombatFeedbackText["MISS"]
		end
	elseif ( event == "BLOCK" ) then
		fontHeight = fontHeight * 0.75
		text = CombatFeedbackText[event]
	elseif ( event == "HEAL" ) then
		text = "+"..amount
		r = 0.0
		g = 1.0
		b = 0.0
		if ( flags == "CRITICAL" ) then
			fontHeight = fontHeight * 1.3
		end
	elseif ( event == "ENERGIZE" ) then
		text = amount
		r = 0.41
		g = 0.8
		b = 0.94
		if ( flags == "CRITICAL" ) then
			fontHeight = fontHeight * 1.3
		end
	else
		text = CombatFeedbackText[event]
	end

	self.feedbackStartTime = GetTime()
	local font = self.HitIndicator:GetFont()
	self.HitIndicator:SetFont(font,fontHeight,"OUTLINE")
	self.HitIndicator:SetText(text)
	self.HitIndicator:SetTextColor(r, g, b)
	self.HitIndicator:SetAlpha(0)
	self.HitIndicator:Show()
	
	feedback[self] = true
end
