local media = LibStub("LibSharedMedia-3.0")
local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")
local newFrame, delFrame = aUF.newFrame, aUF.delFrame

local aUF_healthbar = aUF:NewModule("Healthbar")
aUF_healthbar.hideFromMenu = true

local barColor = {}
local activeframes = {}
local DURATION = 0.5
local fadeFrame = CreateFrame("Frame")
local _abs, _cos, _pi = math.abs, math.cos, math.pi

local function CosineInterpolate(y1, y2, mu)
	local mu2 = (1-_cos(mu*_pi))/2
	return y1*(1-mu2)+y2*mu2
end

local function updateFaders(self)
	local now, stop, alpha = GetTime()
	local ran = false
	for frame, object in pairs(activeframes) do
		ran = true
		stop, alpha = frame.stop
		if stop < now then
			-- done.  set both bar values to the dest value, nuke entry in activeframes, set alpha to 0
			activeframes[frame] = nil
			-- set skipFade == true
			object:UpdateHealth(true)
			frame.startValue = nil
			frame.destValue = nil
			frame:Hide()
			return
		end
		if frame.startValue < frame.destValue then
			alpha = (1 - ((stop - now) / DURATION))*0.7
		else
			alpha = ((stop - now) / DURATION)*0.7
		end
		if frame.style == "flash" then
			frame:SetAlpha(alpha)
		elseif frame.style == 'both' then
			frame:SetAlpha(alpha)
			local cVal = frame:GetValue()
			cVal = CosineInterpolate(cVal,frame.destValue, 1 - ((stop - now) / DURATION) )
			frame:SetValue(cVal)
		elseif frame.style == 'smooth' then
			local cVal = frame:GetValue()
			cVal = CosineInterpolate(cVal,frame.destValue, 1 - ((stop - now) / DURATION) )
			frame:SetValue(cVal)
		end
	end
	if not ran then
		self:Hide()
	end
end

function aUF_healthbar:OnRegister()
	self:RegisterDefaults({
		units = {
			["**"] = {
				ClassColorBars = true,
			},
		}
	})

	self.subConfig = "Bars"
end

function aUF_healthbar:OnRegisterEvents(object)
	if self:IsEnabled() then
		object:ObjectRegisterEvent("UNIT_HEALTH", "UpdateHealth")
		object:ObjectRegisterEvent("UNIT_MAXHEALTH", "UpdateHealth")
		if object.inherit == "pet" then
			object:ObjectRegisterEvent("UNIT_HAPPINESS","HealthBarColor")
		end
	else
		object:ObjectUnregisterEvent("UNIT_HEALTH", "UpdateHealth")
		object:ObjectUnregisterEvent("UNIT_MAXHEALTH", "UpdateHealth")
		if object.inherit == "pet" then
			object:ObjectUnregisterEvent("UNIT_HAPPINESS","HealthBarColor")
		end	
	end
end

function aUF_healthbar:OnEnable()
	-- start timer for aggro detection
	self.schedule = aUF.ScheduleRepeatingTimer(self, "UpdateAggro", 0.2)
	for _,object in pairs(aUF.units) do
		if object.type and object.unit then
			object:ApplyLayout()
			self:OnRegisterEvents(object)
			object:HealthBarColor()
		end
	end
end

function aUF_healthbar:OnDisable()
	aUF.CancelTimer( self, self.schedule )
	for _,object in pairs(aUF.units) do
		if object.type then
			self:OnRegisterEvents(object)
			if object.bars.HealthBar then
				object.bars.HealthBar.bg = delFrame(object.bars.HealthBar.bg)
				object.bars.HealthBar = delFrame(object.bars.HealthBar)
			end
			object:ApplyLayout()
		end
	end
end

function aUF_healthbar:OnDisableObject(object)
end

function aUF_healthbar:OnUpdateAll(object)
	object:UpdateHealth()
	object:HealthBarColor()
end

function aUF_healthbar:OnStatusBarsColor(object)
    if object.bars.HealthBar then
        object:HealthBarColor()
    end
end

function aUF_healthbar:OnMetroUpdate(object)
	object:UpdateHealth()
end

function aUF_healthbar:OnSetWidth(object)
	if object.bars.HealthBar then
		object.bars.HealthBar.bg:ClearAllPoints()
		object.bars.HealthBar.bg:SetAllPoints(object.bars.HealthBar)
	end
end

function aUF_healthbar:OnSetBarTexture(object)
	local m = media:Fetch("statusbar", aUF.db.profile.BarStyle)
	if object.bars.HealthBar then
		object.bars.HealthBar:SetStatusBarTexture(m,"BORDER")
	end
	if object.bars.HealthBar.bg then
		object.bars.HealthBar.bg:SetTexture(m,"BORDER")
	end
	if object.healthFade then
		object.healthFade:SetStatusBarTexture(m)
	end
end

function aUF_healthbar:OnParseLayout(object)
	local bars = object.bars
	if not bars.HealthBar then
		bars.HealthBar = newFrame("StatusBar", object.frame)	
		bars.HealthBar.bg = newFrame("Texture", object.frame, "BORDER")
		bars.HealthBar:SetScript("OnValueChanged",function() object:StatusBarsOnValueChanged(arg1) end)
		bars.HealthBar:SetMinMaxValues(0,100)
	end
end

local targets, victims, aggro, oldaggro = {}, {}, {}, {}
function aUF_healthbar:UpdateAggro()
	-- Almost integrally taken from perfectraid
	if not aUF.units then return end

	local unit, unit2
	for aunit,tbl in pairs(aUF.units) do
		unit = tbl.unit
		if unit then
			if not targets[unit] then
				targets[unit] = unit.."target"
				victims[unit] = unit.."targettarget"
				aggro[unit] = 0
				oldaggro[unit] = 0
			end
			
			local target = targets[unit]
			local victim = victims[unit]
		
			if UnitExists(target) and UnitCanAttack(unit, target) and UnitExists( victim ) and UnitCanAttack(victim, target) then
				-- hostile unit with a friendly victim
				for aunit2, tbl2  in pairs(aUF.units) do
					unit2 = tbl2.unit
					if unit2 and UnitIsUnit( unit2, victim ) then
						-- unit is being targeted by a hostile mob
						if not targets[unit2] then
							targets[unit2] = unit2.."target"
							victims[unit2] = unit2.."targettarget"
							aggro[unit2] = 0
							oldaggro[unit2] = 0
						end
						local val = aggro[unit2]
						if val < 20 then
							aggro[unit2] = val + 10
						end
					end
				end
			end
	
			-- Aggro decay (thanks BanzaiLib)
			if aggro[unit] >= 5 then
				aggro[unit] = aggro[unit] - 5
			end

			if ( oldaggro[unit] > 0 and aggro[unit] == 0 ) or ( oldaggro[unit] == 0 and aggro[unit] > 0 ) then
				tbl:HealthBarColor()
			end
			oldaggro[unit] = aggro[unit]
		end
	end
end

local function DrawFader(object)
	local m = media:Fetch("statusbar", aUF.db.profile.BarStyle)
	local healthFade
	if object.healthFade then
		healthFade = object.healthFade
	else
		healthFade = newFrame("StatusBar", object.frame)
	end
	object.healthFade = healthFade
	healthFade.object = object
	healthFade.destValue = nil
	healthFade.faderstop = nil
	healthFade:SetOrientation(object.bars.HealthBar:GetOrientation())
	healthFade:SetAllPoints(object.bars.HealthBar)
	healthFade:SetMinMaxValues(object.bars.HealthBar:GetMinMaxValues())
	healthFade:SetValue(object.bars.HealthBar:GetValue())
	healthFade:SetStatusBarTexture(m)
	healthFade:Hide()
	-- hack in setting the updater function
	fadeFrame:SetScript("OnUpdate", updateFaders)
	return healthFade
end

---------------
-- STATUSBAR --
---------------

function aUF.classes.aUFunit.prototype:UpdateHealth(skipFade)
	local currValue,maxValue = UnitHealth(self.unit),UnitHealthMax(self.unit)
	local perc = currValue/maxValue * 100

	if ( not UnitExists(self.unit) or UnitIsDead(self.unit) or UnitIsGhost(self.unit) or not UnitIsConnected(self.unit) or maxValue == 1) then
		perc = 0
		self.bars.HealthBar:SetValue(0)
	end
	if self.database.HealthDeficit then
		perc = 100 - perc
	end
	local db = self.database
	if skipFade ~= true and (db.SmoothHealth or db.FadeHealth) then
		DURATION = aUF.db.profile.BarFadeTime
		if not self.healthFade then DrawFader(self) end
		local preValue = self.healthFade.destValue or self.bars.HealthBar:GetValue()
		local destValue = (maxValue == 0 and 0) or perc
		if _abs(destValue - preValue) >= 0.015 then
			local healthFade = self.healthFade
			if (not healthFade.startValue) then
				healthFade.startValue = preValue
				isFading = false
			end
			healthFade.stop = GetTime() + DURATION
			healthFade.destValue = destValue
			if (db.SmoothHealth and db.FadeHealth) then
				if preValue < destValue then
					self.bars.HealthBar:SetValue(preValue)
				end
				if (not isFading) then
					healthFade:SetValue(preValue)
				end
				healthFade:SetAlpha(0)
				healthFade.style = 'both'
			elseif db.SmoothHealth then
				if preValue < destValue then
					self.bars.HealthBar:SetValue(preValue)
				end
				if (not isFading) then
					healthFade:SetValue(preValue)
				end
				healthFade:SetAlpha(0.7)
				healthFade.style = "smooth"
			elseif db.FadeHealth then
				if preValue < destValue then
					healthFade:SetAlpha(0)
					self.bars.HealthBar:SetValue(preValue)
					if (not isFading) then
						healthFade:SetValue(destValue)
					end
				else
					if (not isFading) then
						healthFade:SetAlpha(0.7)
						healthFade:SetValue(preValue)
					end
				end
				healthFade.style = "flash"
			end
			healthFade:Show()
			activeframes[healthFade] = self
			fadeFrame:Show()
		end
	end
	self.bars.HealthBar:SetValue(perc)
end

---------------------
-- STATUSBAR COLOR --
---------------------

local color1 = {r=0.88,g=0.17,b=0.29}
local color2 = {r=1,g=0.97,b=0.1}
function aUF.classes.aUFunit.prototype:HealthBarColor()
	local db = aUF.db.profile
	local healthColor
	local _,eClass = UnitClass(self.unit)

	if self.unit:find("target") and not UnitIsFriend(self.unit, "player") and (self.database.TargetShowHostile == true ) then
		healthColor = aUF:UtilFactionColors(self.unit)
		if aUF.db.profile.SmoothHealthBars == true then
			barColor[self] = 2
		else
			barColor[self] = nil
		end
	elseif self.unit == "pet" 
--	and self.flags.HappinessBar == true 
	and eClass ~= "WARLOCK" then
		local happiness = GetPetHappiness()
		if ( happiness == 1 ) then
			healthColor = color1
		elseif ( happiness == 2 ) then
			healthColor = color2
		else
			healthColor = db.HealthColor
		end
		barColor[self] = nil
	elseif UnitIsPlayer(self.unit) and self.database.ClassColorBars == true and RAID_CLASS_COLORS[eClass] then
		healthColor = RAID_CLASS_COLORS[eClass]
		barColor[self] = nil
	else
		healthColor = db.HealthColor
		barColor[self] = 1
	end

	if ( barColor[self] ) then
		self:StatusBarsOnValueChanged(self.bars.HealthBar:GetValue())
	else
		if self.database.AggroHealth and aggro[self.unit] and aggro[self.unit] > 0 then
			self.bars.HealthBar:SetStatusBarColor(1, 0, 0, 1)
		else
			self.bars.HealthBar:SetStatusBarColor(healthColor.r,healthColor.g,healthColor.b, 1)
		end
	end

--	if self.flags.BackgroundBarColor == true then
		self.bars.HealthBar.bg:SetVertexColor(healthColor.r,healthColor.g,healthColor.b,0.25)
--	else
--		self.bars.HealthBar.bg:SetVertexColor(0,0,0,0.25)
--	end
	if self.healthFade then
		self.healthFade:SetStatusBarColor(self.bars.HealthBar:GetStatusBarColor())
	end
end

function aUF.classes.aUFunit.prototype:StatusBarsOnValueChanged(value)
	if barColor[self] == 1 then
		self:SmoothHealthBarOnValueChanged(value)
	elseif barColor[self] == 2 then
		self:SmoothTargetHealthBarOnValueChanged(value,aUF:UtilFactionColors(self.unit))
	end
end

function aUF.classes.aUFunit.prototype:SmoothTargetHealthBarOnValueChanged(value,colortable)
	if ( not value) then
		return
	end
	local r, g, b
	local min, max = self.bars.HealthBar:GetMinMaxValues()
	if ( (value < min) or (value > max) ) then
		return
	end
	if ( (max - min) > 0 ) then
		value = (value - min) / (max - min)
	else
		value = 0
	end

	r = colortable.r*(0.35*value+0.65)
	g = colortable.g*(0.35*value+0.65)
	b = colortable.b*(0.35*value+0.65)
	if self.database.AggroHealth and aggro[self.unit] and aggro[self.unit] > 0 then
		r = 1
		g = 0
		b = 0
	end
	self.bars.HealthBar:SetStatusBarColor(r, g, b)
	if self.healthFade then
		self.healthFade:SetStatusBarColor(r, g, b)
	end
end

function aUF.classes.aUFunit.prototype:SmoothHealthBarOnValueChanged(value)
	if ( not value ) then
		return
	end
	local db = aUF.db.profile
	local r, g, b
	local min, max = self.bars.HealthBar:GetMinMaxValues()
	if ( (value < min) or (value > max) ) then
		return
	end
	if ( (max - min) > 0 ) then
		value = (value - min) / (max - min)
	else
		value = 0
	end
	if(value > 0.5) then
		r = (db.HealthColor.r) + (((1-value) * 2)* (1-(db.HealthColor.r)))
		g = db.HealthColor.g
	else
		r = 1.0
		g = (db.HealthColor.g) - (0.5 - value)* 2 * (db.HealthColor.g)
	end
	b = db.HealthColor.b
	if self.database.AggroHealth and aggro[self.unit] and aggro[self.unit] > 0 then
		r = 1
		g = 0
		b = 0
	end
	self.bars.HealthBar:SetStatusBarColor(r, g, b)
	if self.healthFade then
		self.healthFade:SetStatusBarColor(r, g, b)
	end
end
