local media = LibStub("LibSharedMedia-3.0")
local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")
local newFrame, delFrame = aUF.newFrame, aUF.delFrame

local aUF_powerbar = aUF:NewModule("Powerbar")

local enabledObjects = {}
local activeframes = {}
local DURATION = 0.5
local fadeFrame = CreateFrame("Frame")
local _abs, _cos, _pi = math.abs, math.cos, math.pi

local function CosineInterpolate(y1, y2, mu)
	local mu2 = (1-_cos(mu*_pi))/2
	return y1*(1-mu2)+y2*mu2
end

local function updateFaders(self)
	local now, stop, alpha = GetTime()
	local ran = false
	for frame, object in pairs(activeframes) do
		ran = true
		stop, alpha = frame.stop
		if stop < now then
			-- done.  set both bar values to the dest value, nuke entry in activeframes, set alpha to 0
			activeframes[frame] = nil
			-- set skipFade == true
			object:UpdatePower(true)
			frame.startValue = nil
			frame.destValue = nil
			frame:Hide()
			return
		end
		if frame.startValue < frame.destValue then
			alpha = (1 - ((stop - now) / DURATION))*0.7
		else
			alpha = ((stop - now) / DURATION)*0.7
		end
		if frame.style == "flash" then
			frame:SetAlpha(alpha)
		elseif frame.style == 'both' then
			frame:SetAlpha(alpha)
			local cVal = frame:GetValue()
			cVal = CosineInterpolate(cVal,frame.destValue, 1 - ((stop - now) / DURATION) )
			frame:SetValue(cVal)
		elseif frame.style == 'smooth' then
			local cVal = frame:GetValue()
			cVal = CosineInterpolate(cVal,frame.destValue, 1 - ((stop - now) / DURATION) )
			frame:SetValue(cVal)
		end
	end
	if not ran then
		self:Hide()
	end
end

function aUF_powerbar:OnRegister()
	self:RegisterDefaults({
		units = {
			["**"] = {
				ShowMana = true,
			},
			partypet = {
				ShowMana = false,
			},
			raid = {
				ShowMana = false,
			},
			raidpet = {
				ShowMana = false,
			},
			targettarget = {
				ShowMana = false,
			},
			targettargettarget = {
				ShowMana = false,
			},
			partytarget = {
				ShowMana = false,
			},
		}
	})

	self.subConfig = "Bars"
end

--aUF.Options.table.args.Units.args.FrameStyle.args.Elements.args
function aUF_powerbar:OnEnable()
	for _,object in pairs(aUF.units) do
		if object.type then
			object:ApplyLayout()
			if object.unit then
				self:OnRegisterEvents(object)			
				object:UpdateAll()
			end
		end
	end
	if not fadeFrame then
		fadeFrame = CreateFrame("Frame")
	end
	fadeFrame:SetScript("OnUpdate", updateFaders)
end

function aUF_powerbar:OnDisable()
	for _,object in pairs(aUF.units) do
		if object.type then
			if object.bars.ManaBar then
				object.bars.ManaBar.bg = delFrame(object.bars.ManaBar.bg)
				object.bars.ManaBar = delFrame(object.bars.ManaBar)
				self:OnRegisterEvents(object)
				object:ApplyLayout()
			end
		end
	end
	if fadeFrame then
		fadeFrame:Hide()
		fadeFrame:SetScript("OnUpdate", nil)
	end
end

function aUF_powerbar:OnDisableObject(object)

end

function aUF_powerbar:OnRegisterEvents(object)
	if (object.database.ShowMana) and self:IsEnabled() then
--		object:ObjectRegisterEvent("UNIT_MAXMANA", "UpdatePower")
		object:ObjectRegisterEvent("UNIT_MANA",	"UpdatePower")
--		object:ObjectRegisterEvent("UNIT_MAXRAGE", "UpdatePower")
		object:ObjectRegisterEvent("UNIT_RAGE", "UpdatePower")
--		object:ObjectRegisterEvent("UNIT_MAXFOCUS", "UpdatePower")
		object:ObjectRegisterEvent("UNIT_FOCUS", "UpdatePower")
--		object:ObjectRegisterEvent("UNIT_MAXENERGY", "UpdatePower")
		object:ObjectRegisterEvent("UNIT_ENERGY", "UpdatePower")
		object:ObjectRegisterEvent("UNIT_DISPLAYPOWER","UpdatePower")
	else
--		object:ObjectUnregisterEvent("UNIT_MAXMANA", "UpdatePower")
		object:ObjectUnregisterEvent("UNIT_MANA", "UpdatePower")
--		object:ObjectUnregisterEvent("UNIT_MAXRAGE", "UpdatePower")
		object:ObjectUnregisterEvent("UNIT_RAGE", "UpdatePower")
--		object:ObjectUnregisterEvent("UNIT_MAXFOCUS", "UpdatePower")
		object:ObjectUnregisterEvent("UNIT_FOCUS", "UpdatePower")
--		object:ObjectUnregisterEvent("UNIT_MAXENERGY", "UpdatePower")
		object:ObjectUnregisterEvent("UNIT_ENERGY", "UpdatePower")
		object:ObjectUnregisterEvent("UNIT_DISPLAYPOWER","UpdatePower")
	end
end

function aUF_powerbar:OnUpdateAll(object)
	if (object.database.ShowMana) and self:IsEnabled() then
		object:UpdatePower()
	end
end

function aUF_powerbar:OnMetroUpdate(object)
	if (object.database.ShowMana) and self:IsEnabled() then
		object:UpdatePower()
	end
end

function aUF_powerbar:OnStatusBarsColor(object)
    if object.bars.ManaBar then
        object:UpdateColor()
    end
end

function aUF_powerbar:OnSetBarTexture(object)
	local m = media:Fetch("statusbar", aUF.db.profile.BarStyle)
	if object.bars.ManaBar then
		object.bars.ManaBar:SetStatusBarTexture(m)
	end
	if object.bars.ManaBar then
		object.bars.ManaBar.bg:SetTexture(m)
	end
	if object.powerFade then
		object.powerFade:SetStatusBarTexture(m)
	end
end

function aUF_powerbar:OnParseLayout(object)
	if object.database.ShowMana then
		object.powertype = nil
		if not object.bars.ManaBar then
			object.bars.ManaBar = newFrame("StatusBar", object.frame)
			object.bars.ManaBar:SetMinMaxValues(0,100)
			object.bars.ManaBar.bg = newFrame("Texture", object.frame, "BORDER")
			object.bars.ManaBar.bg:SetDrawLayer("BORDER")
			object.bars.ManaBar.bg:ClearAllPoints()
			object.bars.ManaBar.bg:SetAllPoints(object.bars.ManaBar)
		end
	else
		if object.bars.ManaBar then
			object.bars.ManaBar.bg = delFrame(object.bars.ManaBar.bg)
			object.bars.ManaBar = delFrame(object.bars.ManaBar)
		end
	end
end

local function DrawFader(object)
	local m = media:Fetch("statusbar", aUF.db.profile.BarStyle)
	local powerFade
	if object.powerFade then
		powerFade = object.powerFade
	else
		powerFade = newFrame("StatusBar", object.frame)
	end
	object.powerFade = powerFade
	powerFade.object = object
	powerFade.destValue = nil
	powerFade.faderstop = nil
	powerFade:SetOrientation(object.bars.ManaBar:GetOrientation())
	powerFade:SetAllPoints(object.bars.ManaBar)
	powerFade:SetMinMaxValues(object.bars.ManaBar:GetMinMaxValues())
	powerFade:SetValue(object.bars.ManaBar:GetValue())
	powerFade:SetStatusBarTexture(m)
	powerFade:Hide()
	return powerFade
end

---------------
-- STATUSBAR --
---------------

function aUF.classes.aUFunit.prototype:UpdatePower(skipFade)
	local currValue,maxValue = UnitMana(self.unit),UnitManaMax(self.unit)
	local perc = currValue/maxValue * 100
	--DURATION = aUF.
	-- db should be the object db
	local db = self.database
	if skipFade ~= true and (db.SmoothMana or db.FadeMana) then
		DURATION = aUF.db.profile.BarFadeTime
		if not self.powerFade then DrawFader(self) end
		local preValue = self.powerFade.destValue or self.bars.ManaBar:GetValue()
		local destValue = (maxValue == 0 and 0) or perc
		if _abs(destValue - preValue) >= 0.015 then
			local powerFade = self.powerFade
			if (not powerFade.startValue) then
				powerFade.startValue = preValue
				isFading = false
			end
			powerFade.stop = GetTime() + DURATION
			powerFade.destValue = destValue
			if (db.SmoothMana and db.FadeMana) then
				if preValue < destValue then
					self.bars.ManaBar:SetValue(preValue)
				end
				if (not isFading) then
					powerFade:SetValue(preValue)
				end
				powerFade:SetAlpha(0)
				powerFade.style = 'both'
			elseif db.SmoothMana then
				if preValue < destValue then
					self.bars.ManaBar:SetValue(preValue)
				end
				if (not isFading) then
					powerFade:SetValue(preValue)
				end
				powerFade:SetAlpha(0.7)
				powerFade.style = "smooth"
			elseif db.FadeMana then
				if preValue < destValue then
					powerFade:SetAlpha(0)
					self.bars.ManaBar:SetValue(preValue)
					if (not isFading) then
						powerFade:SetValue(destValue)
					end
				else
					if (not isFading) then
						powerFade:SetAlpha(0.7)
						powerFade:SetValue(preValue)
					end
				end
				powerFade.style = "flash"
			end
			powerFade:Show()
			activeframes[powerFade] = self
			fadeFrame:Show()
		end
	end

	if ( not UnitExists(self.unit) or UnitIsDead(self.unit) or UnitIsGhost(self.unit) or not UnitIsConnected(self.unit) or (currValue == 1 and maxValue == 1) or UnitHealthMax(self.unit) == 1 ) then
		self.bars.ManaBar:SetValue(0)
		perc = 0
	else
		if maxValue == 0 then
			self.bars.ManaBar:SetValue(0)
		else
			self.bars.ManaBar:SetValue(perc)
		end
	end
	if force or not (self.powertype) or (self.powertype ~= UnitPowerType(self.unit)) then
		self:UpdateColor()
	end
end

function aUF.classes.aUFunit.prototype:UpdateColor()
    local db = aUF.db.profile
	self.powertype = UnitPowerType(self.unit)
	local info = db.ManaColor[self.powertype]
--	if self.flags.BackgroundBarColor == true then
		self.bars.ManaBar.bg:SetVertexColor(info.r,info.g,info.b,0.25)
--	else
--		self.bars.ManaBar.bg:SetVertexColor(0,0,0,0.25)
--	end
--	if self.flags.AlphaBar == true then
		self.bars.ManaBar:SetStatusBarColor(info.r,info.g,info.b,0.8)
--	else
--		self.bars.ManaBar:SetStatusBarColor(info.r,info.g,info.b,1)
--	end
	if self.powerFade then
		self.powerFade:SetStatusBarColor(self.bars.ManaBar:GetStatusBarColor())
	end
end