if select(2, UnitClass("player")) ~= "SHAMAN" then
	-- Non-shamans suck and cannot stand the awesomeness of this module.
	return
end

local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")
local aUF_totems = aUF:NewModule("Totems")
local newFrame, delFrame = aUF.newFrame, aUF.delFrame

aUF_totems.inherit = "player"

function aUF_totems:OnRegister()
	self:RegisterDefaults({
		units = {
			["player"] = {
				Totems = true,
				TotemTooltips = true,
			},
		}
	})
	self.subConfig = "Bars"
end


function aUF_totems:OnRegisterEvents(object)
	if self:IsEnabled() then
		object:ObjectRegisterEvent("PLAYER_TOTEM_UPDATE", "UpdateAllTotems")
		object:ObjectRegisterEvent("PLAYER_ENTERING_WORLD", "UpdateAllTotems")
	else
		object:ObjectUnregisterEvent("PLAYER_TOTEM_UPDATE", "UpdateAllTotems")
		object:ObjectUnregisterEvent("PLAYER_ENTERING_WORLD", "UpdateAllTotems")
	end
end

local function onclick(frame, mousebutton)
	if mousebutton == "RightButton" then
		DestroyTotem(this.slot)
	end
end

local function onenter()
	GameTooltip_SetDefaultAnchor(GameTooltip, this)
	GameTooltip:SetTotem(this.slot)
end

local function onleave()
	GameTooltip:Hide()
end

function aUF_totems:OnParseLayout(object)
	if not object.database.Totems then return end
	if not object.Totems then
		object.Totems = newFrame("Texture", object.frame, "BORDER")
		object.Totems:SetTexture(0, 0, 0, 1)
		object.Totems:SetVertexColor(0.25, 0.25, 0.25, 0.5)
		object.Totems:SetWidth((MAX_TOTEMS * 15) + 4)
		object.Totems:SetHeight(19)
		object.Totems:Show()
		for i=1,MAX_TOTEMS do
			local name = "Totem" .. i
			local frame = newFrame("Button", object.middle, "OVERLAY")
			frame:SetHeight(15)
			frame:SetWidth(15)
			frame:Hide()
			frame.slot = i
			
			frame.Icon = newFrame("Texture", frame, "BACKGROUND")
			frame.Icon:SetAllPoints(frame)

			frame.Cooldown = newFrame("cooldown", frame)
			frame.Cooldown:Hide()
			frame.Cooldown:SetAllPoints(frame)
			frame.Cooldown:SetReverse(true)
			frame.Cooldown:SetFrameLevel(frame:GetFrameLevel())
			frame.Cooldown:SetAlpha(0.8)
			
			object.Totems[name] = frame
			
			if i > 1 then
				frame:SetPoint("BOTTOMLEFT", object.Totems["Totem"..i-1], "BOTTOMRIGHT", 2, 0)
			else
				frame:SetPoint("BOTTOMLEFT", object.Totems,"BOTTOMLEFT", 2, 2)
			end
		end
	end
end

function aUF_totems:OnDisable(object)
	for _,object in pairs(aUF.units) do
		if object.inhereit == self.inhereit and object.Totems then
			for i=1,MAX_TOTEMS do
				local name = "Totem" .. i
				local frame = object.Totems[name]
				frame.Icon = delFrame(frame.Icon)
				frame.Cooldown = delFrame(frame.cooldown)
				object.Totems[name] = delFrame(frame)
			end
			object.Totems = delFrame(object.Totems)
			self:OnRegisterEvents(object)
		end
	end
end

function aUF_totems:OnEnable()
	for _,object in pairs(aUF.units) do
		if object.type and object.inherit == self.inherit and object.database.Totems then
			self:OnParseLayout(object)
			if object.unit then
				object:UpdateAllTotems()
				self:OnRegisterEvents(object)
			end
		end
	end
end

function aUF_totems:OnUpdateAll(object)
	local tooltips = object.database.TotemTooltips
	for i=1,MAX_TOTEMS do
		local frame = object.Totems["Totem"..i]
		frame:RegisterForClicks("RightButtonUp")
		frame:SetScript("OnClick", onclick)
		frame:SetScript("OnEnter", tooltips and onenter or nil)
		frame:SetScript("OnLeave", tooltips and onleave or nil)
	end
	object:UpdateAllTotems()
end

-- Unit methods --

function aUF.classes.aUFunit.prototype:UpdateTotem(slot)
	if not self.Totems then return end
	
	local frame = self.Totems["Totem"..slot]
	
	local haveTotem, name, startTime, duration, icon = GetTotemInfo(slot)
	if haveTotem and name ~= "" then
		frame.Icon:SetTexture(icon)
		frame.Cooldown:SetCooldown(startTime, duration)
		
		frame:Show()
	else
		frame:Hide()
	end
end

function aUF.classes.aUFunit.prototype:UpdateAllTotems()
	for i=1, MAX_TOTEMS do
		self:UpdateTotem(i)
	end
end
